@@ -6,14 +6,19 @@
  * @license GPL v2+
  */
 #include "detect.h"
+
 #include "uncrustify_types.h"
 #include "chunk_list.h"
 #include "ChunkStack.h"
 #include "prototypes.h"
+
+#include "unc_ctype.h"
+
 #include <cstdio>
 #include <cstdlib>
 #include <cstring>
-#include "unc_ctype.h"
+
+using namespace uncrustify;
 
 
 //! Detect spacing options
@@ -23,25 +28,18 @@ static void detect_space_options(void);
 class sp_votes
 {
 protected:
-   size_t   m_add;
-   size_t   m_remove;
-   size_t   m_force;
-   argval_t *m_av;
+   size_t         m_add    = 0;
+   size_t         m_remove = 0;
+   size_t         m_force  = 0;
+   Option<iarf_e> &m_option;
 
 public:
-   sp_votes(argval_t &av)
-   {
-      m_add    = 0;
-      m_remove = 0;
-      m_force  = 0;
-      m_av     = &av;
-   }
-
+   sp_votes(Option<iarf_e> &opt)
+      : m_option(opt) {}
 
    //! Figure out the result of the vote and maybe update *m_av
    ~sp_votes();
 
-
    void vote(chunk_t *first, chunk_t *second);
 };
 
@@ -85,11 +83,11 @@ sp_votes::~sp_votes()
 
    if (m_remove == 0)
    {
-      *m_av = (m_force > m_add) ? AV_FORCE : AV_ADD;
+      m_option = (m_force > m_add) ? IARF_FORCE : IARF_ADD;
    }
    else if (m_force == 0 && m_add == 0)
    {
-      *m_av = AV_REMOVE;
+      m_option = IARF_REMOVE;
    }
    else
    {
@@ -99,7 +97,7 @@ sp_votes::~sp_votes()
 
 
 // generates "vote_sp_xxx" variable name from uncrustify option name "UO_xxx"
-#define SP_VOTE_VAR(x)    sp_votes vote_ ## x(cpd.settings[UO_ ## x].a)
+#define SP_VOTE_VAR(x)    sp_votes vote_ ## x(options::x)
 
 
 static void detect_space_options(void)
@@ -167,12 +165,12 @@ static void detect_space_options(void)
          break;
       }
 
-      if (pc->type == CT_ARITH)
+      if (chunk_is_token(pc, CT_ARITH))
       {
          vote_sp_arith.vote(pc, next);
          vote_sp_arith.vote(prev, pc);
       }
-      if (pc->type == CT_ASSIGN)
+      if (chunk_is_token(pc, CT_ASSIGN))
       {
          if ((pc->flags & PCF_IN_ENUM) == 0)
          {
@@ -185,34 +183,34 @@ static void detect_space_options(void)
             vote_sp_enum_after_assign.vote(pc, next);
          }
       }
-      if (pc->type == CT_SQUARE_OPEN)
+      if (chunk_is_token(pc, CT_SQUARE_OPEN))
       {
          vote_sp_before_square.vote(prev, pc);
          vote_sp_inside_square.vote(pc, next);
       }
-      if (pc->type == CT_SQUARE_CLOSE)
+      if (chunk_is_token(pc, CT_SQUARE_CLOSE))
       {
          vote_sp_inside_square.vote(prev, pc);
       }
-      if (pc->type == CT_TSQUARE)
+      if (chunk_is_token(pc, CT_TSQUARE))
       {
          vote_sp_before_squares.vote(prev, pc);
       }
-      if (pc->type == CT_BOOL)
+      if (chunk_is_token(pc, CT_BOOL))
       {
          vote_sp_bool.vote(prev, pc);
          vote_sp_bool.vote(pc, next);
       }
-      if (pc->type == CT_COMPARE)
+      if (chunk_is_token(pc, CT_COMPARE))
       {
          vote_sp_compare.vote(prev, pc);
          vote_sp_compare.vote(pc, next);
       }
-      if (pc->type == CT_PAREN_CLOSE)
+      if (chunk_is_token(pc, CT_PAREN_CLOSE))
       {
          vote_sp_inside_paren.vote(prev, pc);
       }
-      if (pc->type == CT_PAREN_OPEN)
+      if (chunk_is_token(pc, CT_PAREN_OPEN))
       {
          vote_sp_inside_paren.vote(pc, next);
       }
@@ -221,13 +219,13 @@ static void detect_space_options(void)
       {
          vote_sp_paren_paren.vote(pc, next);
       }
-      if (chunk_is_paren_close(pc) && next->type == CT_BRACE_OPEN)
+      if (chunk_is_paren_close(pc) && chunk_is_token(next, CT_BRACE_OPEN))
       {
          vote_sp_paren_brace.vote(pc, next);
       }
-      if (pc->type == CT_PTR_TYPE)
+      if (chunk_is_token(pc, CT_PTR_TYPE))
       {
-         if (prev->type == CT_PTR_TYPE)
+         if (chunk_is_token(prev, CT_PTR_TYPE))
          {
             vote_sp_between_ptr_star.vote(prev, pc);
          }
@@ -244,7 +242,7 @@ static void detect_space_options(void)
             vote_sp_after_ptr_star.vote(pc, next);
          }
       }
-      if (pc->type == CT_BYREF)
+      if (chunk_is_token(pc, CT_BYREF))
       {
          if (next->type != CT_WORD)
          {
@@ -257,14 +255,14 @@ static void detect_space_options(void)
          vote_sp_after_byref.vote(pc, next);
       }
       if (  pc->type != CT_PTR_TYPE
-         && (prev->type == CT_QUALIFIER || prev->type == CT_TYPE))
+         && (chunk_is_token(prev, CT_QUALIFIER) || chunk_is_token(prev, CT_TYPE)))
       {
          vote_sp_after_type.vote(prev, pc);
       }
-      if (pc->type == CT_ANGLE_OPEN)
+      if (chunk_is_token(pc, CT_ANGLE_OPEN))
       {
          vote_sp_inside_angle.vote(pc, next);
-         if (prev->type == CT_TEMPLATE)
+         if (chunk_is_token(prev, CT_TEMPLATE))
          {
             vote_sp_template_angle.vote(prev, pc);
          }
@@ -273,14 +271,14 @@ static void detect_space_options(void)
             vote_sp_before_angle.vote(prev, pc);
          }
       }
-      if (pc->type == CT_ANGLE_CLOSE)
+      if (chunk_is_token(pc, CT_ANGLE_CLOSE))
       {
          vote_sp_inside_angle.vote(prev, pc);
          if (chunk_is_paren_open(next))
          {
             vote_sp_angle_paren.vote(prev, pc);
          }
-         else if (next->type == CT_WORD || CharTable::IsKw1(next->str[0]))
+         else if (chunk_is_token(next, CT_WORD) || CharTable::IsKw1(next->str[0]))
          {
             vote_sp_angle_word.vote(prev, pc);
          }
@@ -290,15 +288,15 @@ static void detect_space_options(void)
          }
       }
 
-      if (pc->type == CT_SPAREN_OPEN)
+      if (chunk_is_token(pc, CT_SPAREN_OPEN))
       {
          vote_sp_before_sparen.vote(prev, pc);
          vote_sp_inside_sparen.vote(pc, next);
       }
-      if (pc->type == CT_SPAREN_CLOSE)
+      if (chunk_is_token(pc, CT_SPAREN_CLOSE))
       {
          vote_sp_inside_sparen.vote(prev, pc);
-         if (next->type == CT_BRACE_OPEN)
+         if (chunk_is_token(next, CT_BRACE_OPEN))
          {
             vote_sp_sparen_brace.vote(pc, next);
          }
@@ -307,16 +305,16 @@ static void detect_space_options(void)
             vote_sp_after_sparen.vote(pc, next);
          }
       }
-      if (pc->type == CT_SEMICOLON)
+      if (chunk_is_token(pc, CT_SEMICOLON))
       {
          if (pc->parent_type == CT_FOR)
          {
-            if (prev->type == CT_SPAREN_OPEN)
+            if (chunk_is_token(prev, CT_SPAREN_OPEN))
             {
                // empty, ie for (;;)
                vote_sp_before_semi_for_empty.vote(prev, pc);
             }
-            else if (next->type == CT_SPAREN_CLOSE)
+            else if (chunk_is_token(next, CT_SPAREN_CLOSE))
             {
                // empty, ie for (;;)
                vote_sp_after_semi_for_empty.vote(pc, next);
@@ -326,7 +324,7 @@ static void detect_space_options(void)
                vote_sp_before_semi_for.vote(prev, pc);
             }
          }
-         else if (prev->type == CT_VBRACE_OPEN)
+         else if (chunk_is_token(prev, CT_VBRACE_OPEN))
          {
             vote_sp_special_semi.vote(chunk_get_prev(prev), pc);
          }
@@ -335,39 +333,39 @@ static void detect_space_options(void)
             vote_sp_before_semi.vote(prev, pc);
          }
       }
-      if (pc->type == CT_COMMA)
+      if (chunk_is_token(pc, CT_COMMA))
       {
          vote_sp_before_comma.vote(prev, pc);
          vote_sp_after_comma.vote(pc, next);
       }
-      if (pc->type == CT_CLASS_COLON)
+      if (chunk_is_token(pc, CT_CLASS_COLON))
       {
          vote_sp_before_class_colon.vote(prev, pc);
          vote_sp_after_class_colon.vote(pc, next);
       }
-      if (pc->type == CT_BRACE_OPEN)
+      if (chunk_is_token(pc, CT_BRACE_OPEN))
       {
-         if (prev->type == CT_ELSE)
+         if (chunk_is_token(prev, CT_ELSE))
          {
             vote_sp_else_brace.vote(prev, pc);
          }
-         else if (prev->type == CT_CATCH)
+         else if (chunk_is_token(prev, CT_CATCH))
          {
             vote_sp_catch_brace.vote(prev, pc);
          }
-         else if (prev->type == CT_FINALLY)
+         else if (chunk_is_token(prev, CT_FINALLY))
          {
             vote_sp_catch_brace.vote(prev, pc);
          }
-         else if (prev->type == CT_TRY)
+         else if (chunk_is_token(prev, CT_TRY))
          {
             vote_sp_catch_brace.vote(prev, pc);
          }
-         else if (prev->type == CT_GETSET)
+         else if (chunk_is_token(prev, CT_GETSET))
          {
             vote_sp_catch_brace.vote(prev, pc);
          }
-         if (next->type == CT_BRACE_CLOSE)
+         if (chunk_is_token(next, CT_BRACE_CLOSE))
          {
             vote_sp_inside_braces_empty.vote(pc, next);
          }
@@ -376,18 +374,18 @@ static void detect_space_options(void)
             vote_sp_inside_braces.vote(pc, next);
          }
       }
-      if (pc->type == CT_BRACE_CLOSE)
+      if (chunk_is_token(pc, CT_BRACE_CLOSE))
       {
          vote_sp_inside_braces.vote(prev, pc);
-         if (next->type == CT_ELSE)
+         if (chunk_is_token(next, CT_ELSE))
          {
             vote_sp_brace_else.vote(pc, next);
          }
-         else if (next->type == CT_CATCH)
+         else if (chunk_is_token(next, CT_CATCH))
          {
             vote_sp_brace_catch.vote(pc, next);
          }
-         else if (next->type == CT_FINALLY)
+         else if (chunk_is_token(next, CT_FINALLY))
          {
             vote_sp_brace_finally.vote(pc, next);
          }
