@@ -123,12 +123,19 @@ void log_flush(bool force_nl);
 #define LOG_FMT    log_fmt
 // TODO during debugging add source file and line number
 #else
-#define LOG_FMT(sev, args...)                           \
-   do { if (log_sev_on(sev)) { log_fmt(sev, ## args); } \
+#define LOG_FMT(sev, ...)                                   \
+   do { if (log_sev_on(sev)) { log_fmt(sev, __VA_ARGS__); } \
    } while (0)
 #endif
 
 
+#ifdef DEBUG
+#define D_LOG_FMT    LOG_FMT
+#else
+#define D_LOG_FMT(sev, ...)    ((void)0) //forces semicolon after macro
+#endif
+
+
 /**
  * Dumps hex characters inline, no newlines inserted
  *
@@ -173,7 +180,7 @@ void log_hex_blk(log_sev_t sev, const void *data, size_t len);
  * @return '0', '1', '2', '3', '4', '5', '6', '7','8', '9',
  *         'a', 'b', 'c', 'd', 'e', or 'f'
  */
-static_inline char to_hex_char(int nibble)
+static inline char to_hex_char(int nibble)
 {
    const char *hex_string = "0123456789abcdef";
 
