@@ -68,6 +68,7 @@ enum c_token_t
    CT_ENUM,
    CT_ENUM_CLASS,
    CT_SIZEOF,
+   CT_DECLTYPE,
    CT_RETURN,
    CT_BREAK,
    CT_UNION,
@@ -79,6 +80,7 @@ enum c_token_t
    CT_TYPE_CAST,           // static_cast<type>(exp)
    CT_TYPENAME,            // typename type
    CT_TEMPLATE,            // template<...>
+   CT_WHERE_SPEC,          // 'where' : used in C# generic constraint
 
    CT_ASSIGN,              // =, +=, /=, etc
    CT_ASSIGN_NL,           // Assign followed by a newline - fake item for indenting
@@ -132,13 +134,17 @@ enum c_token_t
    CT_CONSTR_COLON,        // colon after a constructor
    CT_D_ARRAY_COLON,       // D named array initializer colon
    CT_COND_COLON,          // conditional colon in  'b ? t : f'
+   CT_WHERE_COLON,         // C# where-constraint colon (after the type)
    CT_QUESTION,
    CT_COMMA,
 
    CT_ASM,
    CT_ATTRIBUTE,
+   CT_OC_AVAILABLE,
+   CT_OC_AVAILABLE_VALUE,
    CT_CATCH,
    CT_WHEN,
+   CT_WHERE,            // C# where clause
    CT_CLASS,
    CT_DELETE,
    CT_EXPORT,
@@ -156,6 +162,7 @@ enum c_token_t
    CT_BRACED_INIT_LIST,
    CT_USING,
    CT_USING_STMT,       // using (xxx) ...
+   CT_USING_ALIAS,      // using identifier attr(optional) = type-id
    CT_D_WITH,           // D: paren+braced
    CT_D_MODULE,
    CT_SUPER,
@@ -167,6 +174,7 @@ enum c_token_t
    CT_UNITTEST,
    CT_UNSAFE,
    CT_FINALLY,
+   CT_FIXED,            // C# fixed
    CT_IMPORT,
    CT_D_SCOPE,
    CT_D_SCOPE_IF,
@@ -182,7 +190,7 @@ enum c_token_t
    CT_ANGLE_OPEN,       // template<T*>
    CT_ANGLE_CLOSE,
 
-   CT_SPAREN_OPEN,      // 'special' paren after if/for/switch/while/synchronized
+   CT_SPAREN_OPEN,      // 'special' paren after if/for/switch/while/synchronized/catch
    CT_SPAREN_CLOSE,
 
    CT_FPAREN_OPEN,      // 'function' paren after fcn/macro fcn
@@ -216,6 +224,7 @@ enum c_token_t
    CT_FUNC_TYPE,        // function type - foo in "typedef void (*foo)(void)"
    CT_FUNC_VAR,         // foo and parent type of first parens in "void (*foo)(void)"
    CT_FUNC_PROTO,       // function prototype
+   CT_FUNC_START,       // global DC member for functions(void ::func())
    CT_FUNC_CLASS_DEF,   // ctor or dtor for a class
    CT_FUNC_CLASS_PROTO, // ctor or dtor for a class
    CT_FUNC_CTOR_VAR,    // variable or class initialization
@@ -225,6 +234,7 @@ enum c_token_t
    CT_MACRO,            // a macro def
    CT_QUALIFIER,        // static, const, etc
    CT_EXTERN,           // extern
+   CT_DECLSPEC,         // __declspec
    CT_ALIGN,            // paren'd qualifier: align(4) struct a { }
    CT_TYPE,
    CT_PTR_TYPE,         // a '*' as part of a type
@@ -326,6 +336,7 @@ enum c_token_t
    CT_SQL_BEGIN,        // the 'BEGINN' in 'EXEC SQL BEGIN ...'
    CT_SQL_END,          // the 'END' in 'EXEC SQL END ...'
    CT_SQL_WORD,         // CT_WORDs in the 'EXEC SQL' statement
+   CT_SQL_ASSIGN,       // :=
 
    // Vala stuff
    CT_CONSTRUCT,        // braced "construct { }" or qualifier "(construct int x)"
@@ -337,6 +348,10 @@ enum c_token_t
    CT_FOR_COLON,    // colon in "for ( TYPE var: EXPR ) { ... }"
    CT_DOUBLE_BRACE, // parent for double brace
 
+   /* Clang */
+   CT_CNG_HASINC,   // Clang: __has_include()
+   CT_CNG_HASINCN,  // Clang: __has_include_next()
+
    // extensions for Qt macros
    CT_Q_EMIT,       // guy 2015-10-16
    CT_Q_FOREACH,    // guy 2015-09-23
