@@ -15,12 +15,17 @@
 #include <cstdio>
 #include <cstdlib>
 #include <cstring>
+#include <stack>
 #include "unc_ctype.h"
 #include "uncrustify.h"
 #include "keywords.h"
+#include "language_tools.h"
+#include "punctuators.h"
 
+#define LE_COUNT(x)    cpd.le_counts[static_cast<size_t>(LE_ ## x)]
 
 using namespace std;
+using namespace uncrustify;
 
 
 struct tok_info
@@ -101,7 +106,7 @@ struct tok_ctx
          switch (ch)
          {
          case '\t':
-            c.col = calc_next_tab_column(c.col, cpd.settings[UO_input_tab_size].u);
+            c.col = calc_next_tab_column(c.col, options::input_tab_size());
             break;
 
          case '\n':
@@ -140,8 +145,8 @@ struct tok_ctx
 
 
    const deque<int> &data;
-   tok_info          c; //! current
-   tok_info          s; //! saved
+   tok_info         c; //! current
+   tok_info         s; //! saved
 };
 
 
@@ -168,18 +173,6 @@ static bool parse_string(tok_ctx &ctx, chunk_t &pc, size_t quote_idx, bool allow
 static bool parse_cs_string(tok_ctx &ctx, chunk_t &pc);
 
 
-/**
- * Interpolated strings start with $" end with a single "
- * Double quotes are escaped by doubling.
- * Need to track embedded { } pairs and ignore anything between.
- *
- * @param pc  The structure to update, str is an input.
- *
- * @return Whether a string was parsed
- */
-static bool parse_cs_interpolated_string(tok_ctx &ctx, chunk_t &pc);
-
-
 /**
  * VALA verbatim string, ends with three quotes (""")
  *
@@ -484,8 +477,8 @@ static const char *str_search(const char *needle, const char *haystack, int hays
 
 static bool parse_comment(tok_ctx &ctx, chunk_t &pc)
 {
-   bool   is_d    = (cpd.lang_flags & LANG_D) != 0;          // forcing value to bool
-   bool   is_cs   = (cpd.lang_flags & LANG_CS) != 0;         // forcing value to bool
+   bool   is_d    = language_is_set(LANG_D);
+   bool   is_cs   = language_is_set(LANG_CS);
    size_t d_level = 0;
 
    // does this start with '/ /' or '/ *' or '/ +' (d)
@@ -587,17 +580,17 @@ static bool parse_comment(tok_ctx &ctx, chunk_t &pc)
             {
                if (ctx.peek() == '\n')
                {
-                  cpd.le_counts[LE_CRLF]++;
+                  ++LE_COUNT(CRLF);
                   pc.str.append(ctx.get());  // store the '\n'
                }
                else
                {
-                  cpd.le_counts[LE_CR]++;
+                  ++LE_COUNT(CR);
                }
             }
             else
             {
-               cpd.le_counts[LE_LF]++;
+               ++LE_COUNT(LF);
             }
          }
       }
@@ -641,17 +634,17 @@ static bool parse_comment(tok_ctx &ctx, chunk_t &pc)
             {
                if (ctx.peek() == '\n')
                {
-                  cpd.le_counts[LE_CRLF]++;
+                  ++LE_COUNT(CRLF);
                   pc.str.append(ctx.get());  // store the '\n'
                }
                else
                {
-                  cpd.le_counts[LE_CR]++;
+                  ++LE_COUNT(CR);
                }
             }
             else
             {
-               cpd.le_counts[LE_LF]++;
+               ++LE_COUNT(LF);
             }
          }
       }
@@ -659,29 +652,21 @@ static bool parse_comment(tok_ctx &ctx, chunk_t &pc)
 
    if (cpd.unc_off)
    {
-      const char *ontext = cpd.settings[UO_enable_processing_cmt].str;
-      if (ontext == nullptr || ontext[0] == '\0')
+      const auto &ontext = options::enable_processing_cmt();
+      if (!ontext.empty() && pc.str.find(ontext.c_str()) >= 0)
       {
-         ontext = UNCRUSTIFY_ON_TEXT;
-      }
-
-      if (pc.str.find(ontext) >= 0)
-      {
-         LOG_FMT(LBCTRL, "Found '%s' on line %zu\n", ontext, pc.orig_line);
+         LOG_FMT(LBCTRL, "Found '%s' on line %zu\n",
+                 ontext.c_str(), pc.orig_line);
          cpd.unc_off = false;
       }
    }
    else
    {
-      const char *offtext = cpd.settings[UO_disable_processing_cmt].str;
-      if (offtext == nullptr || !offtext[0])
+      const auto &offtext = options::disable_processing_cmt();
+      if (!offtext.empty() && pc.str.find(offtext.c_str()) >= 0)
       {
-         offtext = UNCRUSTIFY_OFF_TEXT;
-      }
-
-      if (pc.str.find(offtext) >= 0)
-      {
-         LOG_FMT(LBCTRL, "Found '%s' on line %zu\n", offtext, pc.orig_line);
+         LOG_FMT(LBCTRL, "Found '%s' on line %zu\n",
+                 offtext.c_str(), pc.orig_line);
          cpd.unc_off = true;
          // Issue #842
          cpd.unc_off_used = true;
@@ -841,9 +826,11 @@ static bool parse_number(tok_ctx &ctx, chunk_t &pc)
    /*
     * Check for Hex, Octal, or Binary
     * Note that only D, C++14 and Pawn support binary
+    * Fixes the issue # 1591
+    * In c# the numbers starting with 0 are not treated as octal numbers.
     */
    bool did_hex = false;
-   if (ctx.peek() == '0')
+   if (ctx.peek() == '0' && !language_is_set(LANG_CS))
    {
       size_t  ch;
       chunk_t pc_temp;
@@ -927,6 +914,25 @@ static bool parse_number(tok_ctx &ctx, chunk_t &pc)
    // Check if we stopped on a decimal point & make sure it isn't '..'
    if ((ctx.peek() == '.') && (ctx.peek(1) != '.'))
    {
+      // Issue #1265, 5.clamp()
+      tok_info ss;
+      ctx.save(ss);
+      while (ctx.more() && CharTable::IsKw2(ctx.peek(1)))
+      {
+         // skip characters to check for paren open
+         ctx.get();
+      }
+      if (ctx.peek(1) == '(')
+      {
+         ctx.restore(ss);
+         pc.type = CT_NUMBER;
+         return(true);
+      }
+      else
+      {
+         ctx.restore(ss);
+      }
+
       pc.str.append(ctx.get());
       is_float = true;
       if (did_hex)
@@ -1016,10 +1022,10 @@ static bool parse_number(tok_ctx &ctx, chunk_t &pc)
 
 static bool parse_string(tok_ctx &ctx, chunk_t &pc, size_t quote_idx, bool allow_escape)
 {
-   size_t escape_char        = cpd.settings[UO_string_escape_char].u;
-   size_t escape_char2       = cpd.settings[UO_string_escape_char2].u;
-   bool   should_escape_tabs = (  cpd.settings[UO_string_replace_tab_chars].b
-                               && (cpd.lang_flags & LANG_ALLC));
+   size_t escape_char        = options::string_escape_char();
+   size_t escape_char2       = options::string_escape_char2();
+   bool   should_escape_tabs = (  options::string_replace_tab_chars()
+                               && language_is_set(LANG_ALLC));
 
    pc.str.clear();
    while (quote_idx-- > 0)
@@ -1086,54 +1092,212 @@ static bool parse_string(tok_ctx &ctx, chunk_t &pc, size_t quote_idx, bool allow
    return(true);
 } // parse_string
 
+enum cs_string_t
+{
+   CS_STRING_NONE         = 0,
+   CS_STRING_STRING       = 1 << 0,    // is any kind of string
+   CS_STRING_VERBATIM     = 1 << 1,    // @"" style string
+   CS_STRING_INTERPOLATED = 1 << 2,    // $"" or $@"" style string
+};
 
+static cs_string_t operator |=(cs_string_t &value, cs_string_t other)
+{
+   return(value = static_cast<cs_string_t>(value | other));
+}
+
+
+static cs_string_t parse_cs_string_start(tok_ctx &ctx, chunk_t &pc)
+{
+   cs_string_t stringType = CS_STRING_NONE;
+   int         offset     = 0;
+
+   if (ctx.peek(offset) == '$')
+   {
+      stringType |= CS_STRING_INTERPOLATED;
+      ++offset;
+   }
+
+   if (ctx.peek(offset) == '@')
+   {
+      stringType |= CS_STRING_VERBATIM;
+      ++offset;
+   }
+
+   if (ctx.peek(offset) == '"')
+   {
+      stringType |= CS_STRING_STRING;
+
+      pc.type = CT_STRING;
+
+      for (int i = 0; i <= offset; ++i)
+      {
+         pc.str.append(ctx.get());
+      }
+   }
+   else
+   {
+      stringType = CS_STRING_NONE;
+   }
+
+   return(stringType);
+} // parse_cs_string_start
+
+
+struct CsStringParseState
+{
+   cs_string_t type;
+   int         braceDepth;
+
+
+   CsStringParseState(cs_string_t stringType)
+   {
+      type       = stringType;
+      braceDepth = 0;
+   }
+};
+
+
+/**
+ * C# strings are complex enough (mostly due to interpolation and nesting) that they need a custom parser.
+ */
 static bool parse_cs_string(tok_ctx &ctx, chunk_t &pc)
 {
-   pc.str = ctx.get();
-   pc.str.append(ctx.get());
-   pc.type = CT_STRING;
+   cs_string_t stringType = parse_cs_string_start(ctx, pc);
 
-   bool should_escape_tabs = cpd.settings[UO_string_replace_tab_chars].b;
+   if (stringType == 0)
+   {
+      return(false);
+   }
+
+   // an interpolated string can contain {expressions}, which can contain $"strings", which in turn
+   // can contain {expressions}, so we must track both as they are interleaved, in order to properly
+   // parse the outermost string.
+
+   std::stack<CsStringParseState> parseState; // each entry is a nested string
+   parseState.push(CsStringParseState(stringType));
+
+   bool should_escape_tabs = options::string_replace_tab_chars();
 
-   // go until we hit a zero (end of file) or a single "
    while (ctx.more())
    {
-      size_t ch = ctx.get();
+      if (parseState.top().braceDepth > 0)
+      {
+         // all we can do when in an expr is look for expr close with }, or a new string opening. must do this first
+         // so we can peek and potentially consume chars for new string openings, before the ch=get() happens later,
+         // which is needed for newline processing.
+
+         if (ctx.peek() == '}')
+         {
+            pc.str.append(ctx.get());
+
+            if (ctx.peek() == '}')
+            {
+               pc.str.append(ctx.get()); // in interpolated string, `}}` is escape'd `}`
+            }
+            else
+            {
+               --parseState.top().braceDepth;
+            }
+
+            continue;
+         }
+
+         stringType = parse_cs_string_start(ctx, pc);
+         if (stringType)
+         {
+            parseState.push(CsStringParseState(stringType));
+            continue;
+         }
+      }
+
+      int lastcol = ctx.c.col;
+      int ch      = ctx.get();
+
       pc.str.append(ch);
-      if ((ch == '\n') || (ch == '\r'))
+
+      if (ch == '\n')
       {
          pc.type = CT_STRING_MULTI;
          pc.nl_count++;
       }
-      else if (ch == '\t')
+      else if (ch == '\r')
       {
-         if (should_escape_tabs && !cpd.warned_unable_string_replace_tab_chars)
+         pc.type = CT_STRING_MULTI;
+      }
+      else if (parseState.top().braceDepth > 0)
+      {
+         // do nothing. if we're in a brace, we only want the newline handling, and skip the rest.
+      }
+      else if ((ch == '\t') && should_escape_tabs)
+      {
+         if (parseState.top().type & CS_STRING_VERBATIM)
          {
-            cpd.warned_unable_string_replace_tab_chars = true;
-
-            log_sev_t warnlevel = static_cast<log_sev_t>(cpd.settings[UO_warn_level_tabs_found_in_verbatim_string_literals].u);
-
-            /*
-             * a tab char can't be replaced with \\t because escapes don't
-             * work in here-strings. best we can do is warn.
-             */
-            LOG_FMT(warnlevel, "%s:%zu Detected non-replaceable tab char in literal string\n",
-                    cpd.filename.c_str(), pc.orig_line);
-            if (warnlevel < LWARN)
+            if (!cpd.warned_unable_string_replace_tab_chars)
             {
-               cpd.error_count++;
+               cpd.warned_unable_string_replace_tab_chars = true;
+
+               log_sev_t warnlevel = (log_sev_t)options::warn_level_tabs_found_in_verbatim_string_literals();
+
+               /*
+                * a tab char can't be replaced with \\t because escapes don't
+                * work in here-strings. best we can do is warn.
+                */
+               LOG_FMT(warnlevel, "%s(%d): %s: orig_line is %zu, orig_col is %zu, Detected non-replaceable tab char in literal string\n",
+                       __func__, __LINE__, cpd.filename.c_str(), pc.orig_line, pc.orig_col);
+               LOG_FMT(warnlevel, "%s(%d): Warning is given if doing tab-to-\\t replacement and we have found one in a C# verbatim string literal.\n",
+                       __func__, __LINE__);
+               if (warnlevel < LWARN)
+               {
+                  cpd.error_count++;
+               }
             }
          }
+         else
+         {
+            ctx.c.col = lastcol + 2;
+            pc.str.pop_back(); // remove \t
+            pc.str.append("\\t");
+
+            continue;
+         }
+      }
+      else if (ch == '\\' && !(parseState.top().type & CS_STRING_VERBATIM))
+      {
+         // catch escaped quote in order to avoid ending string (but also must handle \\ to avoid accidental 'escape' seq of `\\"`)
+         if (ctx.peek() == '"' || ctx.peek() == '\\')
+         {
+            pc.str.append(ctx.get());
+         }
       }
       else if (ch == '"')
       {
-         if (ctx.peek() == '"')
+         if ((parseState.top().type & CS_STRING_VERBATIM) && (ctx.peek() == '"'))
          {
+            // in verbatim string, `""` is escape'd `"`
             pc.str.append(ctx.get());
          }
          else
          {
-            break;
+            // end of string
+            parseState.pop();
+            if (parseState.empty())
+            {
+               break;
+            }
+         }
+      }
+      else if (parseState.top().type & CS_STRING_INTERPOLATED)
+      {
+         if (ch == '{')
+         {
+            if (ctx.peek() == '{')
+            {
+               pc.str.append(ctx.get()); // in interpolated string, `{{` is escape'd `{`
+            }
+            else
+            {
+               ++parseState.top().braceDepth;
+            }
          }
       }
    }
@@ -1142,68 +1306,6 @@ static bool parse_cs_string(tok_ctx &ctx, chunk_t &pc)
 } // parse_cs_string
 
 
-static bool parse_cs_interpolated_string(tok_ctx &ctx, chunk_t &pc)
-{
-   pc.str = ctx.get();        // '$'
-   pc.str.append(ctx.get());  // '"'
-   pc.type = CT_STRING;
-
-   int depth = 0;
-
-   // go until we hit a zero (end of file) or a single "
-   while (ctx.more())
-   {
-      size_t ch = ctx.get();
-      pc.str.append(ch);
-
-      // if we are inside a { }, then we only look for a }
-      if (depth > 0)
-      {
-         if (ch == '}')
-         {
-            if (ctx.peek() == '}')
-            {
-               // }} doesn't decrease the depth
-               pc.str.append(ctx.get());  // '{'
-            }
-            else
-            {
-               depth--;
-            }
-         }
-      }
-      else
-      {
-         if (ch == '{')
-         {
-            if (ctx.peek() == '{')
-            {
-               // {{ doesn't increase the depth
-               pc.str.append(ctx.get());
-            }
-            else
-            {
-               depth++;
-            }
-         }
-         else if (ch == '"')
-         {
-            if (ctx.peek() == '"')
-            {
-               pc.str.append(ctx.get());
-            }
-            else
-            {
-               break;
-            }
-         }
-      }
-   }
-
-   return(true);
-} // parse_cs_interpolated_string
-
-
 static void parse_verbatim_string(tok_ctx &ctx, chunk_t &pc)
 {
    pc.type = CT_STRING;
@@ -1365,7 +1467,7 @@ static bool parse_word(tok_ctx &ctx, chunk_t &pc, bool skipcheck)
       else
       {
          pc.type = CT_MACRO;
-         if (cpd.settings[UO_pp_ignore_define_body].b)
+         if (options::pp_ignore_define_body())
          {
             /*
              * We are setting the PP_IGNORE preproc state because the following
@@ -1378,7 +1480,7 @@ static bool parse_word(tok_ctx &ctx, chunk_t &pc, bool skipcheck)
    else
    {
       // '@interface' is reserved, not an interface itself
-      if (  (cpd.lang_flags & LANG_JAVA)
+      if (  language_is_set(LANG_JAVA)
          && pc.str.startswith("@")
          && !pc.str.equals(intr_txt))
       {
@@ -1405,6 +1507,84 @@ static bool parse_word(tok_ctx &ctx, chunk_t &pc, bool skipcheck)
 } // parse_word
 
 
+static size_t parse_attribute_specifier_sequence(tok_ctx &ctx)
+{
+   size_t nested = 0;
+   size_t offset = 0;
+   size_t parens = 0;
+   auto   ch1    = ctx.peek(offset++);
+
+   while (ch1)
+   {
+      auto ch2 = ctx.peek(offset++);
+      while (ch2 == ' ' || ch2 == '\n' || ch2 == '\r' || ch2 == '\t')
+      {
+         ch2 = ctx.peek(offset++);
+      }
+      if (nested == 0 && ch2 != '[')
+      {
+         break;
+      }
+      if (ch1 == '(')
+      {
+         ++parens;
+         ch1 = ch2;
+         continue;
+      }
+      if (ch1 == ')')
+      {
+         if (parens == 0)
+         {
+            break;
+         }
+         --parens;
+         ch1 = ch2;
+         continue;
+      }
+      if (ch1 != '[' && ch1 != ']')
+      {
+         ch1 = ch2;
+         continue;
+      }
+      if (ch2 != ch1)
+      {
+         if (parens == 0)
+         {
+            break;
+         }
+         ch1 = ch2;
+         continue;
+      }
+      if (ch1 == '[')
+      {
+         if (nested != 0 && parens == 0)
+         {
+            break;
+         }
+         ++nested;
+      }
+      else if (--nested == 0)
+      {
+         return(offset);
+      }
+      ch1 = ctx.peek(offset++);
+   }
+   return(0);
+} // parse_attribute_specifier_sequence
+
+
+static bool extract_attribute_specifier_sequence(tok_ctx &ctx, chunk_t &pc, size_t length)
+{
+   pc.str.clear();
+   while (length--)
+   {
+      pc.str.append(ctx.get());
+   }
+   pc.type = CT_ATTRIBUTE;
+   return(true);
+} // extract_attribute_specifier_sequence
+
+
 static bool parse_whitespace(tok_ctx &ctx, chunk_t &pc)
 {
    size_t nl_count = 0;
@@ -1420,12 +1600,12 @@ static bool parse_whitespace(tok_ctx &ctx, chunk_t &pc)
          if (ctx.expect('\n'))
          {
             // CRLF ending
-            cpd.le_counts[LE_CRLF]++;
+            ++LE_COUNT(CRLF);
          }
          else
          {
             // CR ending
-            cpd.le_counts[LE_CR]++;
+            ++LE_COUNT(CR);
          }
          nl_count++;
          pc.orig_prev_sp = 0;
@@ -1433,13 +1613,13 @@ static bool parse_whitespace(tok_ctx &ctx, chunk_t &pc)
 
       case '\n':
          // LF ending
-         cpd.le_counts[LE_LF]++;
+         ++LE_COUNT(LF);
          nl_count++;
          pc.orig_prev_sp = 0;
          break;
 
       case '\t':
-         pc.orig_prev_sp += calc_next_tab_column(cpd.column, cpd.settings[UO_input_tab_size].u) - cpd.column;
+         pc.orig_prev_sp += calc_next_tab_column(cpd.column, options::input_tab_size()) - cpd.column;
          break;
 
       case ' ':
@@ -1574,12 +1754,8 @@ static bool parse_ignored(tok_ctx &ctx, chunk_t &pc)
       return(false);
    }
    // Note that we aren't actually making sure this is in a comment, yet
-   const char *ontext = cpd.settings[UO_enable_processing_cmt].str;
-   if (ontext == nullptr || ontext[0] == '\0')
-   {
-      ontext = UNCRUSTIFY_ON_TEXT;
-   }
-   if (pc.str.find(ontext) < 0)
+   const auto &ontext = options::enable_processing_cmt();
+   if (!ontext.empty() && pc.str.find(ontext.c_str()) < 0)
    {
       pc.type = CT_IGNORED;
       return(true);
@@ -1659,6 +1835,13 @@ static bool parse_next(tok_ctx &ctx, chunk_t &pc)
       {
          size_t ch = ctx.peek();
 
+         // Fix for issue #1752
+         // Ignoring extra spaces after ' \ ' for preproc body continuations
+         if (last == '\\' && ch == ' ')
+         {
+            ctx.get();
+            continue;
+         }
          if ((ch == '\n') || (ch == '\r'))
          {
             // Back off if this is an escaped newline
@@ -1704,33 +1887,22 @@ static bool parse_next(tok_ctx &ctx, chunk_t &pc)
       return(true);
    }
 
-   // Check for C# literal strings, ie @"hello" and identifiers @for
-   if ((cpd.lang_flags & LANG_CS) && (ctx.peek() == '@'))
+   if (language_is_set(LANG_CS))
    {
-      if (ctx.peek(1) == '"')
+      if (parse_cs_string(ctx, pc))
       {
-         parse_cs_string(ctx, pc);
          return(true);
       }
       // check for non-keyword identifiers such as @if @switch, etc
-      if (CharTable::IsKw1(ctx.peek(1)))
+      if ((ctx.peek() == '@') && CharTable::IsKw1(ctx.peek(1)))
       {
          parse_word(ctx, pc, true);
          return(true);
       }
    }
 
-   // Check for C# Interpolated strings
-   if (  (cpd.lang_flags & LANG_CS)
-      && (ctx.peek() == '$')
-      && (ctx.peek(1) == '"'))
-   {
-      parse_cs_interpolated_string(ctx, pc);
-      return(true);
-   }
-
    // handle VALA """ strings """
-   if (  (cpd.lang_flags & LANG_VALA)
+   if (  language_is_set(LANG_VALA)
       && (ctx.peek() == '"')
       && (ctx.peek(1) == '"')
       && (ctx.peek(2) == '"'))
@@ -1744,7 +1916,7 @@ static bool parse_next(tok_ctx &ctx, chunk_t &pc)
     * possible combinations and optional R delimiters: R"delim(x)delim"
     */
    auto ch = ctx.peek();
-   if (  (cpd.lang_flags & LANG_CPP)
+   if (  language_is_set(LANG_C | LANG_CPP)
       && (  ch == 'u'
          || ch == 'U'
          || ch == 'R'
@@ -1761,11 +1933,14 @@ static bool parse_next(tok_ctx &ctx, chunk_t &pc)
       {
          idx++;
       }
-      if (ctx.peek(idx) == 'R')
+
+      if (  language_is_set(LANG_C | LANG_CPP)
+         && ctx.peek(idx) == 'R')
       {
          idx++;
          is_real = true;
       }
+
       const auto quote = ctx.peek(idx);
 
       if (is_real)
@@ -1783,7 +1958,7 @@ static bool parse_next(tok_ctx &ctx, chunk_t &pc)
    }
 
    // PAWN specific stuff
-   if (cpd.lang_flags & LANG_PAWN)
+   if (language_is_set(LANG_PAWN))
    {
       if (  cpd.preproc_ncnl_count == 1
          && (cpd.in_preproc == CT_PP_DEFINE || cpd.in_preproc == CT_PP_EMIT))
@@ -1830,7 +2005,7 @@ static bool parse_next(tok_ctx &ctx, chunk_t &pc)
       return(true);
    }
 
-   if (cpd.lang_flags & LANG_D)
+   if (language_is_set(LANG_D))
    {
       // D specific stuff
       if (d_parse_string(ctx, pc))
@@ -1857,17 +2032,27 @@ static bool parse_next(tok_ctx &ctx, chunk_t &pc)
 
       if ((ch == '<') && cpd.in_preproc == CT_PP_DEFINE)
       {
-         if (chunk_get_tail()->type == CT_MACRO)
+         if (chunk_is_token(chunk_get_tail(), CT_MACRO))
          {
             // We have "#define XXX <", assume '<' starts an include string
             parse_string(ctx, pc, 0, false);
             return(true);
          }
       }
+
+      /* Inside clang's __has_include() could be "path/to/file.h" or system-style <path/to/file.h> */
+      if (  (ch == '(')
+         && (chunk_get_tail() != nullptr)
+         && (  chunk_is_token(chunk_get_tail(), CT_CNG_HASINC)
+            || chunk_is_token(chunk_get_tail(), CT_CNG_HASINCN)))
+      {
+         parse_string(ctx, pc, 0, false);
+         return(true);
+      }
    }
 
    // Check for Objective C literals and VALA identifiers ('@1', '@if')
-   if ((cpd.lang_flags & (LANG_OC | LANG_VALA)) && (ctx.peek() == '@'))
+   if (language_is_set(LANG_OC | LANG_VALA) && (ctx.peek() == '@'))
    {
       size_t nc = ctx.peek(1);
       if ((nc == '"') || (nc == '\''))
@@ -1895,6 +2080,16 @@ static bool parse_next(tok_ctx &ctx, chunk_t &pc)
       return(true);
    }
 
+   // Check for C++11/14/17/20 attribute specifier sequences
+   if (language_is_set(LANG_CPP) && ctx.peek() == '[')
+   {
+      if (auto length = parse_attribute_specifier_sequence(ctx))
+      {
+         extract_attribute_specifier_sequence(ctx, pc, length);
+         return(true);
+      }
+   }
+
    // see if we have a punctuator
    char punc_txt[7];
    punc_txt[0] = ctx.peek();
@@ -1917,6 +2112,32 @@ static bool parse_next(tok_ctx &ctx, chunk_t &pc)
       return(true);
    }
 
+   /* When parsing C/C++ files and running into some unknown token,
+    * check if matches Objective-C as a last resort, before
+    * considering it as garbage.
+    */
+   int probe_lang_flags = 0;
+   if (language_is_set(LANG_C | LANG_CPP))
+   {
+      probe_lang_flags = cpd.lang_flags | LANG_OC;
+   }
+
+   if (probe_lang_flags != 0)
+   {
+      if ((punc = find_punctuator(punc_txt, probe_lang_flags)) != NULL)
+      {
+         cpd.lang_flags = probe_lang_flags;
+         int cnt = strlen(punc->tag);
+         while (cnt--)
+         {
+            pc.str.append(ctx.get());
+         }
+         pc.type   = punc->type;
+         pc.flags |= PCF_PUNCTUATOR;
+         return(true);
+      }
+   }
+
    // throw away this character
    pc.type = CT_UNKNOWN;
    pc.str.append(ctx.get());
@@ -2006,7 +2227,7 @@ void tokenize(const deque<int> &data, chunk_t *ref)
          chunk_flags_set(pc, rprev->flags & PCF_COPY_FLAGS);
 
          // a newline can't be in a preprocessor
-         if (pc->type == CT_NEWLINE)
+         if (chunk_is_token(pc, CT_NEWLINE))
          {
             chunk_flags_clr(pc, PCF_IN_PREPROC);
          }
@@ -2022,14 +2243,14 @@ void tokenize(const deque<int> &data, chunk_t *ref)
       pc = chunk_add_before(&chunk, ref);
 
       // A newline marks the end of a preprocessor
-      if (pc->type == CT_NEWLINE) // || pc->type == CT_COMMENT_MULTI)
+      if (chunk_is_token(pc, CT_NEWLINE)) // || chunk_is_token(pc, CT_COMMENT_MULTI))
       {
          cpd.in_preproc         = CT_NONE;
          cpd.preproc_ncnl_count = 0;
       }
 
       // Disable indentation when #asm directive found
-      if (pc->type == CT_PP_ASM)
+      if (chunk_is_token(pc, CT_PP_ASM))
       {
          LOG_FMT(LBCTRL, "Found a directive %s on line %zu\n", "#asm", pc->orig_line);
          cpd.unc_off = true;
@@ -2067,15 +2288,15 @@ void tokenize(const deque<int> &data, chunk_t *ref)
          }
          else if (cpd.in_preproc == CT_PP_IGNORE)
          {
-            // ASSERT(cpd.settings[UO_pp_ignore_define_body].b);
+            // ASSERT(options::pp_ignore_define_body());
             if (pc->type != CT_NL_CONT && pc->type != CT_COMMENT_CPP)
             {
                set_chunk_type(pc, CT_PP_IGNORE);
             }
          }
          else if (  cpd.in_preproc == CT_PP_DEFINE
-                 && pc->type == CT_PAREN_CLOSE
-                 && cpd.settings[UO_pp_ignore_define_body].b)
+                 && chunk_is_token(pc, CT_PAREN_CLOSE)
+                 && options::pp_ignore_define_body())
          {
             // When we have a PAREN_CLOSE in a PP_DEFINE we should be terminating a MACRO_FUNC
             // arguments list. Therefore we can enter the PP_IGNORE state and ignore next chunks.
@@ -2085,18 +2306,23 @@ void tokenize(const deque<int> &data, chunk_t *ref)
       else
       {
          // Check for a preprocessor start
-         if (  pc->type == CT_POUND
-            && (rprev == nullptr || rprev->type == CT_NEWLINE))
+         if (  chunk_is_token(pc, CT_POUND)
+            && (rprev == nullptr || chunk_is_token(rprev, CT_NEWLINE)))
          {
             set_chunk_type(pc, CT_PREPROC);
             pc->flags     |= PCF_IN_PREPROC;
             cpd.in_preproc = CT_PREPROC;
          }
       }
-      if (pc->type == CT_NEWLINE)
+      if (chunk_is_token(pc, CT_NEWLINE))
       {
-         LOG_FMT(LGUY, "%s(%d): orig_line is %zu, orig_col is %zu, <NewLine>\n",
-                 __func__, __LINE__, pc->orig_line, pc->orig_col);
+         LOG_FMT(LGUY, "%s(%d): orig_line is %zu, orig_col is %zu, <Newline>, nl is %zu\n",
+                 __func__, __LINE__, pc->orig_line, pc->orig_col, pc->nl_count);
+      }
+      else if (chunk_is_token(pc, CT_VBRACE_OPEN))
+      {
+         LOG_FMT(LGUY, "%s(%d): orig_line is %zu, orig_col is %zu, type is %s, orig_col_end is %zu\n",
+                 __func__, __LINE__, pc->orig_line, pc->orig_col, get_token_name(pc->type), pc->orig_col_end);
       }
       else
       {
@@ -2106,23 +2332,23 @@ void tokenize(const deque<int> &data, chunk_t *ref)
    }
 
    // Set the cpd.newline string for this file
-   if (  cpd.settings[UO_newlines].le == LE_LF
-      || (  cpd.settings[UO_newlines].le == LE_AUTO
-         && (cpd.le_counts[LE_LF] >= cpd.le_counts[LE_CRLF])
-         && (cpd.le_counts[LE_LF] >= cpd.le_counts[LE_CR])))
+   if (  options::newlines() == LE_LF
+      || (  options::newlines() == LE_AUTO
+         && (LE_COUNT(LF) >= LE_COUNT(CRLF))
+         && (LE_COUNT(LF) >= LE_COUNT(CR))))
    {
       // LF line ends
       cpd.newline = "\n";
       LOG_FMT(LLINEENDS, "Using LF line endings\n");
    }
-   else if (  cpd.settings[UO_newlines].le == LE_CRLF
-           || (  cpd.settings[UO_newlines].le == LE_AUTO
-              && (cpd.le_counts[LE_CRLF] >= cpd.le_counts[LE_LF])
-              && (cpd.le_counts[LE_CRLF] >= cpd.le_counts[LE_CR])))
+   else if (  options::newlines() == LE_CRLF
+           || (  options::newlines() == LE_AUTO
+              && (LE_COUNT(CRLF) >= LE_COUNT(LF))
+              && (LE_COUNT(CRLF) >= LE_COUNT(CR))))
    {
       // CRLF line ends
       cpd.newline = "\r\n";
-      LOG_FMT(LLINEENDS, "Using CRLF line endings\n");
+      LOG_FMT(LLINEENDS, "Using CRLF line endings\r\n");
    }
    else
    {
