@@ -20,8 +20,33 @@
 #include "space.h"
 #include "combine.h"
 #include "keywords.h"
+#include "language_tools.h"
+#include "punctuators.h"
+
+#include <vector>
+
 #include <cstring>
 
+using namespace uncrustify;
+
+
+/**
+ * Mark types in a single template argument.
+ *
+ * @param start  chunk to start check at
+ * @param end    chunk to end check at
+ */
+static void check_template_arg(chunk_t *start, chunk_t *end);
+
+
+/**
+ * Mark types in template argument(s).
+ *
+ * @param start  chunk to start check at
+ * @param end    chunk to end check at
+ */
+static void check_template_args(chunk_t *start, chunk_t *end);
+
 
 /**
  * If there is nothing but CT_WORD and CT_MEMBER, then it's probably a
@@ -68,8 +93,8 @@ static chunk_t *handle_double_angle_close(chunk_t *pc)
 
    if (next)
    {
-      if (  pc->type == CT_ANGLE_CLOSE
-         && next->type == CT_ANGLE_CLOSE
+      if (  chunk_is_token(pc, CT_ANGLE_CLOSE)
+         && chunk_is_token(next, CT_ANGLE_CLOSE)
          && pc->parent_type == CT_NONE
          && (pc->orig_col_end + 1) == next->orig_col
          && next->parent_type == CT_NONE)
@@ -131,7 +156,7 @@ void tokenize_cleanup(void)
    chunk_t *pc;
    for (pc = chunk_get_head(); pc != nullptr; pc = chunk_get_next_ncnl(pc))
    {
-      if (pc->type == CT_SQUARE_OPEN)
+      if (chunk_is_token(pc, CT_SQUARE_OPEN))
       {
          next = chunk_get_next_ncnl(pc);
          if (chunk_is_token(next, CT_SQUARE_CLOSE))
@@ -148,7 +173,7 @@ void tokenize_cleanup(void)
             chunk_del(next);
          }
       }
-      if (  pc->type == CT_SEMICOLON
+      if (  chunk_is_token(pc, CT_SEMICOLON)
          && (pc->flags & PCF_IN_PREPROC)
          && !chunk_get_next_ncnl(pc, scope_e::PREPROC))
       {
@@ -157,25 +182,42 @@ void tokenize_cleanup(void)
       }
    }
 
+   // change := to CT_SQL_ASSIGN Issue #527
+   for (pc = chunk_get_head(); pc != nullptr; pc = chunk_get_next_ncnl(pc))
+   {
+      if (chunk_is_token(pc, CT_COLON))
+      {
+         next = chunk_get_next_ncnl(pc);
+         if (chunk_is_token(next, CT_ASSIGN))
+         {
+            // Change ':' + '=' into ':='
+            set_chunk_type(pc, CT_SQL_ASSIGN);
+            pc->str          = ":=";
+            pc->orig_col_end = next->orig_col_end;
+            chunk_del(next);
+         }
+      }
+   }
+
    // We can handle everything else in the second pass
    pc   = chunk_get_head();
    next = chunk_get_next_ncnl(pc);
    while (pc != nullptr && next != nullptr)
    {
-      if (pc->type == CT_DOT && (cpd.lang_flags & LANG_ALLC))
+      if (chunk_is_token(pc, CT_DOT) && language_is_set(LANG_ALLC))
       {
          set_chunk_type(pc, CT_MEMBER);
       }
 
-      if (pc->type == CT_NULLCOND && (cpd.lang_flags & LANG_CS))
+      if (chunk_is_token(pc, CT_NULLCOND) && language_is_set(LANG_CS))
       {
          set_chunk_type(pc, CT_MEMBER);
       }
 
       // Determine the version stuff (D only)
-      if (pc->type == CT_D_VERSION)
+      if (chunk_is_token(pc, CT_D_VERSION))
       {
-         if (next->type == CT_PAREN_OPEN)
+         if (chunk_is_token(next, CT_PAREN_OPEN))
          {
             set_chunk_type(pc, CT_D_VERSION_IF);
          }
@@ -192,9 +234,9 @@ void tokenize_cleanup(void)
       }
 
       // Determine the scope stuff (D only)
-      if (pc->type == CT_D_SCOPE)
+      if (chunk_is_token(pc, CT_D_SCOPE))
       {
-         if (next->type == CT_PAREN_OPEN)
+         if (chunk_is_token(next, CT_PAREN_OPEN))
          {
             set_chunk_type(pc, CT_D_SCOPE_IF);
          }
@@ -207,13 +249,17 @@ void tokenize_cleanup(void)
       /*
        * Change CT_BASE before CT_PAREN_OPEN to CT_WORD.
        * public myclass() : base() {}
+       * -or-
+       * var x = (T)base.y;
        */
-      if (pc->type == CT_BASE && next->type == CT_PAREN_OPEN)
+      if (  chunk_is_token(pc, CT_BASE)
+         && (chunk_is_token(next, CT_PAREN_OPEN) || chunk_is_token(next, CT_DOT)))
       {
          set_chunk_type(pc, CT_WORD);
       }
 
-      if (pc->type == CT_ENUM && next->type == CT_CLASS)
+      if (  chunk_is_token(pc, CT_ENUM)
+         && chunk_is_token(next, CT_CLASS))
       {
          set_chunk_type(next, CT_ENUM_CLASS);
       }
@@ -222,16 +268,16 @@ void tokenize_cleanup(void)
        * Change CT_WORD after CT_ENUM, CT_UNION, or CT_STRUCT to CT_TYPE
        * Change CT_WORD before CT_WORD to CT_TYPE
        */
-      if (next->type == CT_WORD)
+      if (chunk_is_token(next, CT_WORD))
       {
-         if (  pc->type == CT_ENUM
-            || pc->type == CT_ENUM_CLASS
-            || pc->type == CT_UNION
-            || pc->type == CT_STRUCT)
+         if (  chunk_is_token(pc, CT_ENUM)
+            || chunk_is_token(pc, CT_ENUM_CLASS)
+            || chunk_is_token(pc, CT_UNION)
+            || chunk_is_token(pc, CT_STRUCT))
          {
             set_chunk_type(next, CT_TYPE);
          }
-         if (pc->type == CT_WORD)
+         if (chunk_is_token(pc, CT_WORD))
          {
             set_chunk_type(pc, CT_TYPE);
          }
@@ -241,13 +287,13 @@ void tokenize_cleanup(void)
        * change extern to qualifier if extern isn't followed by a string or
        * an open parenthesis
        */
-      if (pc->type == CT_EXTERN)
+      if (chunk_is_token(pc, CT_EXTERN))
       {
-         if (next->type == CT_STRING)
+         if (chunk_is_token(next, CT_STRING))
          {
             // Probably 'extern "C"'
          }
-         else if (next->type == CT_PAREN_OPEN)
+         else if (chunk_is_token(next, CT_PAREN_OPEN))
          {
             // Probably 'extern (C)'
          }
@@ -268,38 +314,33 @@ void tokenize_cleanup(void)
        * or by a
        *     CT_WORD which is preceded by CT_DC_MEMBER: '::aaa *b'
        */
-      if (next->type == CT_STAR)
+      if (  (chunk_is_token(next, CT_STAR))
+         || (language_is_set(LANG_CPP) && (chunk_is_token(next, CT_CARET)))
+         || (language_is_set(LANG_CS) && (chunk_is_token(next, CT_QUESTION)) && (strcmp(pc->text(), "null") != 0)))
       {
-         if (  pc->type == CT_TYPE
-            || pc->type == CT_QUALIFIER
-            || pc->type == CT_PTR_TYPE)
+         if (  chunk_is_token(pc, CT_TYPE)
+            || chunk_is_token(pc, CT_QUALIFIER)
+            || chunk_is_token(pc, CT_PTR_TYPE))
          {
             set_chunk_type(next, CT_PTR_TYPE);
          }
-         else if (  pc->type == CT_WORD
-                 && prev != nullptr
-                 && prev->type == CT_DC_MEMBER
-                 && (cpd.lang_flags & LANG_CPP) != 0)
-         {
-            set_chunk_type(pc, CT_TYPE);
-            set_chunk_type(next, CT_PTR_TYPE);
-         }
       }
 
-      if (pc->type == CT_TYPE_CAST && next->type == CT_ANGLE_OPEN)
+      if (  chunk_is_token(pc, CT_TYPE_CAST)
+         && chunk_is_token(next, CT_ANGLE_OPEN))
       {
          set_chunk_parent(next, CT_TYPE_CAST);
          in_type_cast = true;
       }
 
       // Change angle open/close to CT_COMPARE, if not a template thingy
-      if (pc->type == CT_ANGLE_OPEN && pc->parent_type != CT_TYPE_CAST)
+      if (chunk_is_token(pc, CT_ANGLE_OPEN) && pc->parent_type != CT_TYPE_CAST)
       {
          /*
           * pretty much all languages except C use <> for something other than
           * comparisons.  "#include<xxx>" is handled elsewhere.
           */
-         if (cpd.lang_flags & (LANG_CPP | LANG_CS | LANG_JAVA | LANG_VALA | LANG_OC))
+         if (language_is_set(LANG_OC | LANG_CPP | LANG_CS | LANG_JAVA | LANG_VALA))
          {
             // bug #663
             check_template(pc);
@@ -310,7 +351,7 @@ void tokenize_cleanup(void)
             set_chunk_type(pc, CT_COMPARE);
          }
       }
-      if (pc->type == CT_ANGLE_CLOSE && pc->parent_type != CT_TEMPLATE)
+      if (chunk_is_token(pc, CT_ANGLE_CLOSE) && pc->parent_type != CT_TEMPLATE)
       {
          if (in_type_cast)
          {
@@ -323,38 +364,36 @@ void tokenize_cleanup(void)
          }
       }
 
-      if (cpd.lang_flags & LANG_D)
+      if (language_is_set(LANG_D))
       {
          // Check for the D string concat symbol '~'
-         if (  pc->type == CT_INV
-            && (  prev->type == CT_STRING
-               || prev->type == CT_WORD
-               || next->type == CT_STRING))
+         if (  chunk_is_token(pc, CT_INV)
+            && (  chunk_is_token(prev, CT_STRING)
+               || chunk_is_token(prev, CT_WORD)
+               || chunk_is_token(next, CT_STRING)))
          {
             set_chunk_type(pc, CT_CONCAT);
          }
 
          // Check for the D template symbol '!' (word + '!' + word or '(')
-         if (  pc->type == CT_NOT
-            && prev->type == CT_WORD
-            && (  next->type == CT_PAREN_OPEN
-               || next->type == CT_WORD
-               || next->type == CT_TYPE))
+         if (  chunk_is_token(pc, CT_NOT)
+            && chunk_is_token(prev, CT_WORD)
+            && (  chunk_is_token(next, CT_PAREN_OPEN)
+               || chunk_is_token(next, CT_WORD)
+               || chunk_is_token(next, CT_TYPE)))
          {
             set_chunk_type(pc, CT_D_TEMPLATE);
          }
 
          // handle "version(unittest) { }" vs "unittest { }"
-         if (  prev
-            && pc->type == CT_UNITTEST
-            && prev->type == CT_PAREN_OPEN)
+         if (  chunk_is_token(pc, CT_UNITTEST)
+            && chunk_is_token(prev, CT_PAREN_OPEN))
          {
             set_chunk_type(pc, CT_WORD);
          }
 
          // handle 'static if' and merge the tokens
-         if (  prev
-            && pc->type == CT_IF
+         if (  chunk_is_token(pc, CT_IF)
             && chunk_is_str(prev, "static", 6))
          {
             // delete PREV and merge with IF
@@ -364,26 +403,31 @@ void tokenize_cleanup(void)
             pc->orig_line = prev->orig_line;
             chunk_t *to_be_deleted = prev;
             prev = chunk_get_prev_ncnl(prev);
-            chunk_del(to_be_deleted);
+            if (prev != nullptr)
+            {
+               chunk_del(to_be_deleted);
+            }
          }
       }
 
-      if (cpd.lang_flags & LANG_CPP)
+      if (language_is_set(LANG_CPP))
       {
          // Change Word before '::' into a type
-         if (pc->type == CT_WORD && next->type == CT_DC_MEMBER)
+         if (  chunk_is_token(pc, CT_WORD)
+            && chunk_is_token(next, CT_DC_MEMBER))
          {
             set_chunk_type(pc, CT_TYPE);
          }
       }
 
       // Change get/set to CT_WORD if not followed by a brace open
-      if (pc->type == CT_GETSET && next->type != CT_BRACE_OPEN)
+      if (  chunk_is_token(pc, CT_GETSET)
+         && next->type != CT_BRACE_OPEN)
       {
-         if (  next->type == CT_SEMICOLON
-            && (  prev->type == CT_BRACE_CLOSE
-               || prev->type == CT_BRACE_OPEN
-               || prev->type == CT_SEMICOLON))
+         if (  chunk_is_token(next, CT_SEMICOLON)
+            && (  chunk_is_token(prev, CT_BRACE_CLOSE)
+               || chunk_is_token(prev, CT_BRACE_OPEN)
+               || chunk_is_token(prev, CT_SEMICOLON)))
          {
             set_chunk_type(pc, CT_GETSET_EMPTY);
             set_chunk_parent(next, CT_GETSET);
@@ -398,7 +442,7 @@ void tokenize_cleanup(void)
        * Interface is only a keyword in MS land if followed by 'class' or 'struct'
        * likewise, 'class' may be a member name in Java.
        */
-      if (  pc->type == CT_CLASS
+      if (  chunk_is_token(pc, CT_CLASS)
          && !CharTable::IsKw1(next->str[0])
          && pc->next->type != CT_DC_MEMBER)
       {
@@ -422,14 +466,14 @@ void tokenize_cleanup(void)
        * In all cases except the last, this will put the entire operator value
        * in one chunk.
        */
-      if (pc->type == CT_OPERATOR)
+      if (chunk_is_token(pc, CT_OPERATOR))
       {
          chunk_t *tmp2 = chunk_get_next(next);
          // Handle special case of () operator -- [] already handled
-         if (next->type == CT_PAREN_OPEN)
+         if (chunk_is_token(next, CT_PAREN_OPEN))
          {
             chunk_t *tmp = chunk_get_next(next);
-            if (tmp != nullptr && tmp->type == CT_PAREN_CLOSE)
+            if (chunk_is_token(tmp, CT_PAREN_CLOSE))
             {
                next->str = "()";
                set_chunk_type(next, CT_OPERATOR_VAL);
@@ -437,9 +481,8 @@ void tokenize_cleanup(void)
                next->orig_col_end += 1;
             }
          }
-         else if (  next->type == CT_ANGLE_CLOSE
-                 && tmp2
-                 && tmp2->type == CT_ANGLE_CLOSE
+         else if (  chunk_is_token(next, CT_ANGLE_CLOSE)
+                 && chunk_is_token(tmp2, CT_ANGLE_CLOSE)
                  && tmp2->orig_col == next->orig_col_end)
          {
             next->str.append('>');
@@ -500,18 +543,18 @@ void tokenize_cleanup(void)
       }
 
       // Change private, public, protected into either a qualifier or label
-      if (pc->type == CT_PRIVATE)
+      if (chunk_is_token(pc, CT_PRIVATE))
       {
          // Handle Qt slots - maybe should just check for a CT_WORD?
          if (chunk_is_str(next, "slots", 5) || chunk_is_str(next, "Q_SLOTS", 7))
          {
             chunk_t *tmp = chunk_get_next(next);
-            if (tmp != nullptr && tmp->type == CT_COLON)
+            if (chunk_is_token(tmp, CT_COLON))
             {
                next = tmp;
             }
          }
-         if (next->type == CT_COLON)
+         if (chunk_is_token(next, CT_COLON))
          {
             set_chunk_type(next, CT_PRIVATE_COLON);
             chunk_t *tmp;
@@ -530,7 +573,10 @@ void tokenize_cleanup(void)
 
       // Look for <newline> 'EXEC' 'SQL'
       if (  (chunk_is_str_case(pc, "EXEC", 4) && chunk_is_str_case(next, "SQL", 3))
-         || ((*pc->str.c_str() == '$') && pc->type != CT_SQL_WORD))
+         || (  (*pc->str.c_str() == '$') && pc->type != CT_SQL_WORD
+               /* but avoid breaking tokenization for C# 6 interpolated strings. */
+            && (  !language_is_set(LANG_CS)
+               || ((chunk_is_token(pc, CT_STRING)) && (!pc->str.startswith("$\"")) && (!pc->str.startswith("$@\""))))))
       {
          chunk_t *tmp = chunk_get_prev(pc);
          if (chunk_is_newline(tmp))
@@ -573,7 +619,7 @@ void tokenize_cleanup(void)
             // Change words into CT_SQL_WORD until CT_SEMICOLON
             while (tmp != nullptr)
             {
-               if (tmp->type == CT_SEMICOLON)
+               if (chunk_is_token(tmp, CT_SEMICOLON))
                {
                   break;
                }
@@ -589,7 +635,7 @@ void tokenize_cleanup(void)
       }
 
       // handle MS abomination 'for each'
-      if (  pc->type == CT_FOR
+      if (  chunk_is_token(pc, CT_FOR)
          && chunk_is_str(next, "each", 4)
          && (next == chunk_get_next(pc)))
       {
@@ -600,7 +646,7 @@ void tokenize_cleanup(void)
          chunk_del(next);
          next = chunk_get_next_ncnl(pc);
          // label the 'in'
-         if (next && next->type == CT_PAREN_OPEN)
+         if (chunk_is_token(next, CT_PAREN_OPEN))
          {
             chunk_t *tmp = chunk_get_next_ncnl(next);
             while (tmp && tmp->type != CT_PAREN_CLOSE)
@@ -619,34 +665,49 @@ void tokenize_cleanup(void)
        * ObjectiveC allows keywords to be used as identifiers in some situations
        * This is a dirty hack to allow some of the more common situations.
        */
-      if (cpd.lang_flags & LANG_OC)
+      if (language_is_set(LANG_OC))
       {
-         if (  (  pc->type == CT_IF
-               || pc->type == CT_FOR
-               || pc->type == CT_WHILE)
+         if (  (  chunk_is_token(pc, CT_IF)
+               || chunk_is_token(pc, CT_FOR)
+               || chunk_is_token(pc, CT_WHILE))
             && !chunk_is_token(next, CT_PAREN_OPEN))
          {
             set_chunk_type(pc, CT_WORD);
          }
-         if (  pc->type == CT_DO
+         if (  chunk_is_token(pc, CT_DO)
             && (  chunk_is_token(prev, CT_MINUS)
                || chunk_is_token(next, CT_SQUARE_CLOSE)))
          {
             set_chunk_type(pc, CT_WORD);
          }
+
+         // Fix self keyword back to word when mixing c++/objective-c
+         if (  chunk_is_token(pc, CT_THIS) && !strcmp(pc->text(), "self")
+            && (chunk_is_token(next, CT_COMMA) || chunk_is_token(next, CT_PAREN_CLOSE)))
+         {
+            set_chunk_type(pc, CT_WORD);
+         }
+
+         // Fix self keyword back to word when mixing c++/objective-c
+         if (  chunk_is_token(pc, CT_THIS)
+            && !strcmp(pc->text(), "self")
+            && (chunk_is_token(next, CT_COMMA) || chunk_is_token(next, CT_PAREN_CLOSE)))
+         {
+            set_chunk_type(pc, CT_WORD);
+         }
       }
 
       // Another hack to clean up more keyword abuse
-      if (  pc->type == CT_CLASS
+      if (  chunk_is_token(pc, CT_CLASS)
          && (chunk_is_token(prev, CT_DOT) || chunk_is_token(next, CT_DOT)))
       {
          set_chunk_type(pc, CT_WORD);
       }
 
       // Detect Objective C class name
-      if (  pc->type == CT_OC_IMPL
-         || pc->type == CT_OC_INTF
-         || pc->type == CT_OC_PROTOCOL)
+      if (  chunk_is_token(pc, CT_OC_IMPL)
+         || chunk_is_token(pc, CT_OC_INTF)
+         || chunk_is_token(pc, CT_OC_PROTOCOL))
       {
          if (next->type != CT_PAREN_OPEN)
          {
@@ -667,7 +728,7 @@ void tokenize_cleanup(void)
          }
       }
 
-      if (pc->type == CT_OC_INTF)
+      if (chunk_is_token(pc, CT_OC_INTF))
       {
          chunk_t *tmp = chunk_get_next_ncnl(pc, scope_e::PREPROC);
          while (tmp != nullptr && tmp->type != CT_OC_END)
@@ -692,15 +753,15 @@ void tokenize_cleanup(void)
        */
       if (  (  pc->parent_type == CT_OC_IMPL
             || pc->parent_type == CT_OC_INTF
-            || pc->type == CT_OC_CLASS)
-         && next->type == CT_PAREN_OPEN)
+            || chunk_is_token(pc, CT_OC_CLASS))
+         && chunk_is_token(next, CT_PAREN_OPEN))
       {
          set_chunk_parent(next, pc->parent_type);
 
          chunk_t *tmp = chunk_get_next(next);
          if (tmp != nullptr && tmp->next != nullptr)
          {
-            if (tmp->type == CT_PAREN_CLOSE)
+            if (chunk_is_token(tmp, CT_PAREN_CLOSE))
             {
                //set_chunk_type(tmp, CT_OC_CLASS_EXT);
                set_chunk_parent(tmp, pc->parent_type);
@@ -724,7 +785,7 @@ void tokenize_cleanup(void)
        *   @property NSString *stringProperty;
        *   @property(nonatomic, retain) NSMutableDictionary *shareWith;
        */
-      if (pc->type == CT_OC_PROPERTY)
+      if (chunk_is_token(pc, CT_OC_PROPERTY))
       {
          if (next->type != CT_PAREN_OPEN)
          {
@@ -742,7 +803,8 @@ void tokenize_cleanup(void)
        *   @selector(msgNameWith1Arg:)
        *   @selector(msgNameWith2Args:arg2Name:)
        */
-      if (pc->type == CT_OC_SEL && next->type == CT_PAREN_OPEN)
+      if (  chunk_is_token(pc, CT_OC_SEL)
+         && chunk_is_token(next, CT_PAREN_OPEN))
       {
          set_chunk_parent(next, pc->type);
 
@@ -754,7 +816,7 @@ void tokenize_cleanup(void)
 
             while ((tmp = chunk_get_next_ncnl(tmp)) != nullptr)
             {
-               if (tmp->type == CT_PAREN_CLOSE)
+               if (chunk_is_token(tmp, CT_PAREN_CLOSE))
                {
                   set_chunk_parent(tmp, CT_OC_SEL);
                   break;
@@ -766,13 +828,14 @@ void tokenize_cleanup(void)
       }
 
       // Handle special preprocessor junk
-      if (pc->type == CT_PREPROC)
+      if (chunk_is_token(pc, CT_PREPROC))
       {
          set_chunk_parent(pc, next->type);
       }
 
       // Detect "pragma region" and "pragma endregion"
-      if (pc->type == CT_PP_PRAGMA && next->type == CT_PREPROC_BODY)
+      if (  chunk_is_token(pc, CT_PP_PRAGMA)
+         && chunk_is_token(next, CT_PREPROC_BODY))
       {
          if (  (strncmp(next->str.c_str(), "region", 6) == 0)
             || (strncmp(next->str.c_str(), "endregion", 9) == 0))
@@ -784,64 +847,32 @@ void tokenize_cleanup(void)
          }
       }
 
-      // Check for C# nullable types '?' is in next
-      if (  (cpd.lang_flags & LANG_CS)
-         && next->type == CT_QUESTION
-         && (next->orig_col == (pc->orig_col + pc->len())))
-      {
-         chunk_t *tmp = chunk_get_next_ncnl(next);
-         if (tmp != nullptr)
-         {
-            bool doit = (  tmp->type == CT_PAREN_CLOSE
-                        || tmp->type == CT_ANGLE_CLOSE);
-
-            if (tmp->type == CT_WORD)
-            {
-               chunk_t *tmp2 = chunk_get_next_ncnl(tmp);
-               if (  tmp2 != nullptr
-                  && (  tmp2->type == CT_SEMICOLON
-                     || tmp2->type == CT_ASSIGN
-                     || tmp2->type == CT_COMMA
-                     || tmp2->type == CT_BRACE_OPEN))
-               {
-                  doit = true;
-               }
-            }
-
-            if (doit)
-            {
-               pc->str         += next->str;
-               pc->orig_col_end = next->orig_col_end;
-               chunk_del(next);
-               next = tmp;
-            }
-         }
-      }
-
       // Change 'default(' into a sizeof-like statement
-      if (  (cpd.lang_flags & LANG_CS)
-         && pc->type == CT_DEFAULT
-         && next->type == CT_PAREN_OPEN)
+      if (  language_is_set(LANG_CS)
+         && chunk_is_token(pc, CT_DEFAULT)
+         && chunk_is_token(next, CT_PAREN_OPEN))
       {
          set_chunk_type(pc, CT_SIZEOF);
       }
 
-      if (pc->type == CT_UNSAFE && next->type != CT_BRACE_OPEN)
+      if (chunk_is_token(pc, CT_UNSAFE) && next->type != CT_BRACE_OPEN)
       {
          set_chunk_type(pc, CT_QUALIFIER);
       }
 
-      if (  (  pc->type == CT_USING
-            || (pc->type == CT_TRY && (cpd.lang_flags & LANG_JAVA)))
-         && next->type == CT_PAREN_OPEN)
+      if (  (  chunk_is_token(pc, CT_USING)
+            || (chunk_is_token(pc, CT_TRY) && language_is_set(LANG_JAVA)))
+         && chunk_is_token(next, CT_PAREN_OPEN))
       {
          set_chunk_type(pc, CT_USING_STMT);
       }
 
       // Add minimal support for C++0x rvalue references
-      if (pc->type == CT_BOOL && chunk_is_str(pc, "&&", 2))
+      if (  chunk_is_token(pc, CT_BOOL)
+         && language_is_set(LANG_CPP)
+         && chunk_is_str(pc, "&&", 2))
       {
-         if (prev->type == CT_TYPE)
+         if (chunk_is_token(prev, CT_TYPE))
          {
             // Issue # 1002
             if ((pc->flags & PCF_IN_TEMPLATE) == 0)
@@ -855,10 +886,9 @@ void tokenize_cleanup(void)
        * HACK: treat try followed by a colon as a qualifier to handle this:
        *   A::A(int) try : B() { } catch (...) { }
        */
-      if (  pc->type == CT_TRY
+      if (  chunk_is_token(pc, CT_TRY)
          && chunk_is_str(pc, "try", 3)
-         && next != nullptr
-         && next->type == CT_COLON)
+         && chunk_is_token(next, CT_COLON))
       {
          set_chunk_type(pc, CT_QUALIFIER);
       }
@@ -867,16 +897,16 @@ void tokenize_cleanup(void)
        * If Java's 'synchronized' is in a method declaration, it should be
        * a qualifier.
        */
-      if (  (cpd.lang_flags & LANG_JAVA)
-         && pc->type == CT_SYNCHRONIZED
+      if (  language_is_set(LANG_JAVA)
+         && chunk_is_token(pc, CT_SYNCHRONIZED)
          && next->type != CT_PAREN_OPEN)
       {
          set_chunk_type(pc, CT_QUALIFIER);
       }
 
       // change CT_DC_MEMBER + CT_FOR into CT_DC_MEMBER + CT_FUNC_CALL
-      if (  pc->type == CT_FOR
-         && (pc->prev != nullptr && pc->prev->type == CT_DC_MEMBER))
+      if (  chunk_is_token(pc, CT_FOR)
+         && chunk_is_token(pc->prev, CT_DC_MEMBER))
       {
          set_chunk_type(pc, CT_FUNC_CALL);
       }
@@ -891,11 +921,8 @@ void tokenize_cleanup(void)
 
 static void check_template(chunk_t *start)
 {
-   LOG_FMT(LTEMPL, "%s(%d): orig_line %zu, orig_col %zu:",
+   LOG_FMT(LTEMPL, "%s(%d): orig_line %zu, orig_col %zu:\n",
            __func__, __LINE__, start->orig_line, start->orig_col);
-#ifdef DEBUG
-   LOG_FMT(LTEMPL, "\n");
-#endif // DEBUG
 
    chunk_t *prev = chunk_get_prev_ncnl(start, scope_e::PREPROC);
    if (prev == nullptr)
@@ -905,48 +932,49 @@ static void check_template(chunk_t *start)
 
    chunk_t *end;
    chunk_t *pc;
-   if (prev->type == CT_TEMPLATE)
+   if (chunk_is_token(prev, CT_TEMPLATE))
    {
-#ifdef DEBUG
-      LOG_FMT(LTEMPL, "%s(%d):", __func__, __LINE__);
-#endif
-      LOG_FMT(LTEMPL, " CT_TEMPLATE:");
-#ifdef DEBUG
-      LOG_FMT(LTEMPL, "\n");
-#endif
+      LOG_FMT(LTEMPL, "%s(%d): CT_TEMPLATE:\n", __func__, __LINE__);
 
       // We have: "template< ... >", which is a template declaration
-      size_t level = 1;
+      size_t level  = 1;
+      size_t parens = 0;
       for (pc = chunk_get_next_ncnl(start, scope_e::PREPROC);
            pc != nullptr;
            pc = chunk_get_next_ncnl(pc, scope_e::PREPROC))
       {
-         LOG_FMT(LTEMPL, "%s(%d): [%s,%zu]",
+         LOG_FMT(LTEMPL, "%s(%d): type is %s, level is %zu\n",
                  __func__, __LINE__, get_token_name(pc->type), level);
-#ifdef DEBUG
-         LOG_FMT(LTEMPL, "\n");
-#endif
 
          if ((pc->str[0] == '>') && (pc->len() > 1))
          {
-            LOG_FMT(LTEMPL, "%s(%d): {split '%s' at orig_line %zu, orig_col %zu}",
+            LOG_FMT(LTEMPL, "%s(%d): {split '%s' at orig_line %zu, orig_col %zu}\n",
                     __func__, __LINE__, pc->text(), pc->orig_line, pc->orig_col);
-#ifdef DEBUG
-            LOG_FMT(LTEMPL, "\n");
-#endif
             split_off_angle_close(pc);
          }
 
-         if (chunk_is_str(pc, "<", 1))
+         if (pc->type == CT_PAREN_OPEN)
          {
-            level++;
+            ++parens;
          }
-         else if (chunk_is_str(pc, ">", 1))
+         else if (pc->type == CT_PAREN_CLOSE)
          {
-            level--;
-            if (level == 0)
+            --parens;
+         }
+
+         if (parens == 0)
+         {
+            if (chunk_is_str(pc, "<", 1))
             {
-               break;
+               level++;
+            }
+            else if (chunk_is_str(pc, ">", 1))
+            {
+               level--;
+               if (level == 0)
+               {
+                  break;
+               }
             }
          }
       }
@@ -966,37 +994,41 @@ static void check_template(chunk_t *start)
       if (  prev->type != CT_WORD
          && prev->type != CT_TYPE
          && prev->type != CT_COMMA
+         && prev->type != CT_QUALIFIER
          && prev->type != CT_OPERATOR_VAL
          && prev->parent_type != CT_OPERATOR)
       {
-         LOG_FMT(LTEMPL, "%s(%d): - after %s + ( - Not a template\n",
+         LOG_FMT(LTEMPL, "%s(%d): - after type %s + ( - Not a template\n",
                  __func__, __LINE__, get_token_name(prev->type));
-#ifdef DEBUG
-         LOG_FMT(LTEMPL, "\n");
-#endif
          set_chunk_type(start, CT_COMPARE);
          return;
       }
 
-      LOG_FMT(LTEMPL, "%s(%d): - prev %s -",
+      LOG_FMT(LTEMPL, "%s(%d): - prev->type is %s -\n",
               __func__, __LINE__, get_token_name(prev->type));
-#ifdef DEBUG
-      LOG_FMT(LTEMPL, "\n");
-#endif
 
       // Scan back and make sure we aren't inside square parenthesis
-      bool in_if = false;
+      bool in_if         = false;
+      bool hit_semicolon = false;
       pc = start;
       while ((pc = chunk_get_prev_ncnl(pc, scope_e::PREPROC)) != nullptr)
       {
-         if (  pc->type == CT_SEMICOLON
-            || pc->type == CT_BRACE_OPEN
-            || pc->type == CT_BRACE_CLOSE
-            || pc->type == CT_SQUARE_CLOSE)
+         if (  (chunk_is_token(pc, CT_SEMICOLON) && hit_semicolon)
+            || chunk_is_token(pc, CT_BRACE_OPEN)
+            || chunk_is_token(pc, CT_BRACE_CLOSE)
+            || chunk_is_token(pc, CT_SQUARE_CLOSE))
          {
             break;
          }
-         if (pc->type == CT_IF || pc->type == CT_RETURN)
+         if (chunk_is_token(pc, CT_SEMICOLON) && !hit_semicolon)
+         {
+            hit_semicolon = true;
+         }
+         if (  ((  chunk_is_token(pc, CT_IF)
+                || chunk_is_token(pc, CT_RETURN)
+                || chunk_is_token(pc, CT_WHILE)
+                || chunk_is_token(pc, CT_WHILE_OF_DO)) && hit_semicolon == false)
+            || (chunk_is_token(pc, CT_FOR) && hit_semicolon == true))
          {
             in_if = true;
             break;
@@ -1016,23 +1048,19 @@ static void check_template(chunk_t *start)
            pc != nullptr;
            pc = chunk_get_next_ncnl(pc, scope_e::PREPROC))
       {
-         LOG_FMT(LTEMPL, "%s(%d): [%s,%zu]",
+         LOG_FMT(LTEMPL, "%s(%d): type is %s, num_tokens is %zu\n",
                  __func__, __LINE__, get_token_name(pc->type), num_tokens);
-#ifdef DEBUG
-         LOG_FMT(LTEMPL, "\n");
-#endif
 
          if (  (tokens[num_tokens - 1] == CT_ANGLE_OPEN)
             && (pc->str[0] == '>')
             && (pc->len() > 1)
-            && (  cpd.settings[UO_tok_split_gte].b
-               || (chunk_is_str(pc, ">>", 2) && num_tokens >= 2)))
+            && (  options::tok_split_gte()
+               || (  (chunk_is_str(pc, ">>", 2) || chunk_is_str(pc, ">>>", 3))
+                  && num_tokens >= 2)))
          {
-            LOG_FMT(LTEMPL, "%s(%d): {split '%s' at orig_line %zu, orig_col %zu}",
+            LOG_FMT(LTEMPL, "%s(%d): {split '%s' at orig_line %zu, orig_col %zu}\n",
                     __func__, __LINE__, pc->text(), pc->orig_line, pc->orig_col);
-#ifdef DEBUG
-            LOG_FMT(LTEMPL, "\n");
-#endif
+
             split_off_angle_close(pc);
          }
 
@@ -1057,17 +1085,17 @@ static void check_template(chunk_t *start)
             }
          }
          else if (  in_if
-                 && (pc->type == CT_BOOL || pc->type == CT_COMPARE))
+                 && (chunk_is_token(pc, CT_BOOL) || chunk_is_token(pc, CT_COMPARE)))
          {
             break;
          }
-         else if (  pc->type == CT_BRACE_OPEN
-                 || pc->type == CT_BRACE_CLOSE
-                 || pc->type == CT_SEMICOLON)
+         else if (  chunk_is_token(pc, CT_BRACE_OPEN)
+                 || chunk_is_token(pc, CT_BRACE_CLOSE)
+                 || chunk_is_token(pc, CT_SEMICOLON))
          {
             break;
          }
-         else if (pc->type == CT_PAREN_OPEN)
+         else if (chunk_is_token(pc, CT_PAREN_OPEN))
          {
             if (num_tokens >= MAX_NUMBER_OF_TOKEN - 1)
             {
@@ -1076,7 +1104,7 @@ static void check_template(chunk_t *start)
             tokens[num_tokens] = CT_PAREN_OPEN;
             num_tokens++;
          }
-         else if (pc->type == CT_PAREN_CLOSE)
+         else if (chunk_is_token(pc, CT_PAREN_CLOSE))
          {
             num_tokens--;
             if (tokens[num_tokens] != CT_PAREN_OPEN)
@@ -1088,74 +1116,20 @@ static void check_template(chunk_t *start)
       end = pc;
    }
 
-   if (end != nullptr && end->type == CT_ANGLE_CLOSE)
+   if (chunk_is_token(end, CT_ANGLE_CLOSE))
    {
       pc = chunk_get_next_ncnl(end, scope_e::PREPROC);
       if (pc == nullptr || pc->type != CT_NUMBER)
       {
-#ifdef DEBUG
-         LOG_FMT(LTEMPL, "%s(%d):", __func__, __LINE__);
-#endif
-         LOG_FMT(LTEMPL, " - Template Detected\n");
-#ifdef DEBUG
-         LOG_FMT(LTEMPL, "%s(%d):", __func__, __LINE__);
-#endif
-         LOG_FMT(LTEMPL, "     from orig_line %zu, orig_col %zu\n",
-                 start->orig_line, start->orig_col);
-#ifdef DEBUG
-         LOG_FMT(LTEMPL, "%s(%d):", __func__, __LINE__);
-#endif
-         LOG_FMT(LTEMPL, "     to   orig_line %zu, orig_col %zu\n",
-                 end->orig_line, end->orig_col);
-
+         LOG_FMT(LTEMPL, "%s(%d): Template detected\n", __func__, __LINE__);
+         LOG_FMT(LTEMPL, "%s(%d):     from orig_line %zu, orig_col %zu\n",
+                 __func__, __LINE__, start->orig_line, start->orig_col);
+         LOG_FMT(LTEMPL, "%s(%d):     to   orig_line %zu, orig_col %zu\n",
+                 __func__, __LINE__, end->orig_line, end->orig_col);
          set_chunk_parent(start, CT_TEMPLATE);
 
-         // Issue #1127
-         // MyFoo<mySize * 2> foo1;
-         // MyFoo<2*mySize * 2> foo1;
-         // Issue #1346
-         // use it as ONE line:
-         //   typename std::enable_if<!std::is_void<T>::value,
-         //   QVector<T> >::type dummy(const std::function<T*(const S&)>&
-         //   pFunc, const QVector<S>& pItems)
-         // we nees two runs
-         // 1. run to test if expression is numeric
-         bool expressionIsNumeric = false;
-         pc = start;
-         while (pc != end)
-         {
-            chunk_t *next = chunk_get_next_ncnl(pc, scope_e::PREPROC);
-            // a test "if (next == nullptr)" is not necessary
-            chunk_flags_set(pc, PCF_IN_TEMPLATE);
-            if (next->type != CT_PAREN_OPEN)
-            {
-               if (  pc->type == CT_NUMBER
-                  || (pc->type == CT_ARITH && pc->type != CT_STAR))
-               {
-                  expressionIsNumeric = true;
-                  break;
-               }
-            }
-            pc = next;
-         }
-         LOG_FMT(LTEMPL, "%s(%d): expressionIsNumeric is %s\n",
-                 __func__, __LINE__, expressionIsNumeric ? "FALSE" : "TRUE");
-         // 2. run to do the work
-         if (!expressionIsNumeric)
-         {
-            pc = start;
-            while (pc != end)
-            {
-               chunk_t *next = chunk_get_next_ncnl(pc, scope_e::PREPROC);
-               // a test "if (next == nullptr)" is not necessary
-               chunk_flags_set(pc, PCF_IN_TEMPLATE);
-               if (next->type != CT_PAREN_OPEN)
-               {
-                  make_type(pc);
-               }
-               pc = next;
-            }
-         }
+         check_template_args(start, end);
+
          set_chunk_parent(end, CT_TEMPLATE);
          chunk_flags_set(end, PCF_IN_TEMPLATE);
          return;
@@ -1168,9 +1142,122 @@ static void check_template(chunk_t *start)
 } // check_template
 
 
+static void check_template_arg(chunk_t *start, chunk_t *end)
+{
+   LOG_FMT(LTEMPL, "%s(%d): Template argument detected\n", __func__, __LINE__);
+   LOG_FMT(LTEMPL, "%s(%d):     from orig_line %zu, orig_col %zu\n",
+           __func__, __LINE__, start->orig_line, start->orig_col);
+   LOG_FMT(LTEMPL, "%s(%d):     to   orig_line %zu, orig_col %zu\n",
+           __func__, __LINE__, end->orig_line, end->orig_col);
+
+   // Issue #1127
+   // MyFoo<mySize * 2> foo1;
+   // MyFoo<2*mySize * 2> foo1;
+   // Issue #1346
+   // use it as ONE line:
+   //   typename std::enable_if<!std::is_void<T>::value,
+   //   QVector<T> >::type dummy(const std::function<T*(const S&)>&
+   //   pFunc, const QVector<S>& pItems)
+   // we need two runs
+   // 1. run to test if expression is numeric
+   bool    expressionIsNumeric = false;
+   chunk_t *pc                 = start;
+   while (pc != end)
+   {
+      chunk_t *next = chunk_get_next_ncnl(pc, scope_e::PREPROC);
+      // a test "if (next == nullptr)" is not necessary
+      chunk_flags_set(pc, PCF_IN_TEMPLATE);
+      if (chunk_is_token(pc, CT_DECLTYPE) || chunk_is_token(pc, CT_SIZEOF))
+      {
+         expressionIsNumeric = true;
+         break;
+      }
+      if (next->type != CT_PAREN_OPEN)
+      {
+         if (  chunk_is_token(pc, CT_NUMBER)
+            || chunk_is_token(pc, CT_ARITH))
+         {
+            expressionIsNumeric = true;
+            break;
+         }
+      }
+      pc = next;
+   }
+   LOG_FMT(LTEMPL, "%s(%d): expressionIsNumeric is %s\n",
+           __func__, __LINE__, expressionIsNumeric ? "TRUE" : "FALSE");
+   // 2. run to do the work
+   if (!expressionIsNumeric)
+   {
+      pc = start;
+      while (pc != end)
+      {
+         chunk_t *next = chunk_get_next_ncnl(pc, scope_e::PREPROC);
+         // a test "if (next == nullptr)" is not necessary
+         chunk_flags_set(pc, PCF_IN_TEMPLATE);
+         if (next->type != CT_PAREN_OPEN)
+         {
+            make_type(pc);
+         }
+         pc = next;
+      }
+   }
+} // check_template_arg
+
+
+static void check_template_args(chunk_t *start, chunk_t *end)
+{
+   std::vector<c_token_t> tokens;
+
+   // Scan for commas
+   chunk_t *pc;
+
+   for (pc = chunk_get_next_ncnl(start, scope_e::PREPROC);
+        pc != nullptr && pc != end;
+        pc = chunk_get_next_ncnl(pc, scope_e::PREPROC))
+   {
+      switch (pc->type)
+      {
+      case CT_COMMA:
+         if (tokens.empty())
+         {
+            // Check current argument
+            check_template_args(start, pc);
+            start = pc;
+         }
+         break;
+
+      case CT_ANGLE_OPEN:
+      case CT_PAREN_OPEN:
+         tokens.push_back(pc->type);
+         break;
+
+      case CT_ANGLE_CLOSE:
+         if (!tokens.empty() && tokens.back() == CT_ANGLE_OPEN)
+         {
+            tokens.pop_back();
+         }
+         break;
+
+      case CT_PAREN_CLOSE:
+         if (!tokens.empty() && tokens.back() == CT_PAREN_OPEN)
+         {
+            tokens.pop_back();
+         }
+         break;
+
+      default:
+         break;
+      }
+   }
+
+   // Check whatever is left
+   check_template_arg(start, end);
+} // check_template_args
+
+
 static void cleanup_objc_property(chunk_t *start)
 {
-   assert(start && start->type == CT_OC_PROPERTY);
+   assert(chunk_is_token(start, CT_OC_PROPERTY));
 
    chunk_t *open_paren = chunk_get_next_type(start, CT_PAREN_OPEN, start->level);
 
@@ -1205,13 +1292,13 @@ static void cleanup_objc_property(chunk_t *start)
 
 static void mark_selectors_in_property_with_open_paren(chunk_t *open_paren)
 {
-   assert(open_paren && open_paren->type == CT_PAREN_OPEN);
+   assert(chunk_is_token(open_paren, CT_PAREN_OPEN));
 
    chunk_t *tmp = open_paren;
 
    while (tmp && tmp->type != CT_PAREN_CLOSE)
    {
-      if (  tmp->type == CT_WORD
+      if (  chunk_is_token(tmp, CT_WORD)
          && (chunk_is_str(tmp, "setter", 6) || chunk_is_str(tmp, "getter", 6)))
       {
          tmp = tmp->next;
@@ -1219,7 +1306,7 @@ static void mark_selectors_in_property_with_open_paren(chunk_t *open_paren)
                && tmp->type != CT_COMMA
                && tmp->type != CT_PAREN_CLOSE)
          {
-            if (tmp->type == CT_WORD || chunk_is_str(tmp, ":", 1))
+            if (chunk_is_token(tmp, CT_WORD) || chunk_is_str(tmp, ":", 1))
             {
                tmp->type = CT_OC_SEL_NAME;
             }
@@ -1236,15 +1323,14 @@ static void mark_selectors_in_property_with_open_paren(chunk_t *open_paren)
 
 static void mark_attributes_in_property_with_open_paren(chunk_t *open_paren)
 {
-   assert(open_paren && open_paren->type == CT_PAREN_OPEN);
+   assert(chunk_is_token(open_paren, CT_PAREN_OPEN));
 
    chunk_t *tmp = open_paren;
 
    while (tmp && tmp->type != CT_PAREN_CLOSE)
    {
-      if (  (tmp->type == CT_COMMA || tmp->type == CT_PAREN_OPEN)
-         && tmp->next
-         && (tmp->next->type == CT_WORD || tmp->next->type == CT_TYPE))
+      if (  (chunk_is_token(tmp, CT_COMMA) || chunk_is_token(tmp, CT_PAREN_OPEN))
+         && (chunk_is_token(tmp->next, CT_WORD) || chunk_is_token(tmp->next, CT_TYPE)))
       {
          tmp->next->type = CT_OC_PROPERTY_ATTR;
       }
