@@ -12,6 +12,18 @@
 #include "args.h"
 #include "output.h"
 
+#ifdef DEVELOP_ONLY
+/*
+ * the test suite Coveralls: https://coveralls.io
+ * will complains because these functions are only
+ * used at developement time.
+ * Don't worry about unsed lines for the functions:
+ *   prot_the_line
+ *   prot_the_source
+ *   examine_Data
+ *   dump_out
+ *   dump_in
+ */
 
 static size_t counter = 0;
 static size_t tokenCounter;
@@ -33,31 +45,31 @@ void prot_the_line(int theLine, unsigned int actual_line)
       if (pc->orig_line == actual_line)
       {
          tokenCounter++;
-         LOG_FMT(LGUY, " orig_line is %d, ", actual_line);
-         if (pc->type == CT_VBRACE_OPEN)
+         LOG_FMT(LGUY, " orig_line is %d, (%zu) ", actual_line, tokenCounter);
+         if (chunk_is_token(pc, CT_VBRACE_OPEN))
          {
             LOG_FMT(LGUY, "<VBRACE_OPEN>, ");
          }
-         else if (pc->type == CT_NEWLINE)
+         else if (chunk_is_token(pc, CT_NEWLINE))
          {
-            LOG_FMT(LGUY, "<NL>(%zu), ", pc->nl_count);
+            LOG_FMT(LGUY, "<NL>(nl_count is %zu), ", pc->nl_count);
          }
-         else if (pc->type == CT_VBRACE_CLOSE)
+         else if (chunk_is_token(pc, CT_VBRACE_CLOSE))
          {
             LOG_FMT(LGUY, "<CT_VBRACE_CLOSE>, ");
          }
-         else if (pc->type == CT_VBRACE_OPEN)
+         else if (chunk_is_token(pc, CT_VBRACE_OPEN))
          {
             LOG_FMT(LGUY, "<CT_VBRACE_OPEN>, ");
          }
-         else if (pc->type == CT_SPACE)
+         else if (chunk_is_token(pc, CT_SPACE))
          {
             LOG_FMT(LGUY, "<CT_SPACE>, ");
          }
          else
          {
-            LOG_FMT(LGUY, "(%zu) text() %s, type is %s, parent_type is %s, orig_col is %zu, column is %zu, ",
-                    tokenCounter, pc->text(), get_token_name(pc->type), get_token_name(pc->parent_type), pc->orig_col, pc->column);
+            LOG_FMT(LGUY, "text() '%s', column is %zu, type is %s, parent_type is %s, orig_col is %zu, ",
+                    pc->text(), pc->column, get_token_name(pc->type), get_token_name(pc->parent_type), pc->orig_col);
          }
          LOG_FMT(LGUY, "pc->flags:");
          log_pcf_flags(LGUY, pc->flags);
@@ -75,9 +87,6 @@ void prot_the_source(int theLine)
 }
 
 
-// TODO: examine_Data seems not to be used, is it still required?
-
-
 // examples:
 //   examine_Data(__func__, __LINE__, n);
 void examine_Data(const char *func_name, int theLine, int what)
@@ -90,7 +99,7 @@ void examine_Data(const char *func_name, int theLine, int what)
    case 1:
       for (pc = chunk_get_head(); pc != nullptr; pc = pc->next)
       {
-         if (pc->type == CT_SQUARE_CLOSE || pc->type == CT_TSQUARE)
+         if (chunk_is_token(pc, CT_SQUARE_CLOSE) || chunk_is_token(pc, CT_TSQUARE))
          {
             LOG_FMT(LGUY, "\n");
             LOG_FMT(LGUY, "1:(%d),", theLine);
@@ -105,7 +114,7 @@ void examine_Data(const char *func_name, int theLine, int what)
       {
          if (pc->orig_line == 7)
          {
-            if (pc->type == CT_NEWLINE)
+            if (chunk_is_token(pc, CT_NEWLINE))
             {
                LOG_FMT(LGUY, "(%zu)<NL> col=%zu\n\n", pc->orig_line, pc->orig_col);
             }
@@ -121,7 +130,7 @@ void examine_Data(const char *func_name, int theLine, int what)
       LOG_FMT(LGUY, "3:(%d)\n", theLine);
       for (pc = chunk_get_head(); pc != nullptr; pc = pc->next)
       {
-         if (pc->type == CT_NEWLINE)
+         if (chunk_is_token(pc, CT_NEWLINE))
          {
             LOG_FMT(LGUY, "(%zu)<NL> col=%zu\n\n", pc->orig_line, pc->orig_col);
          }
@@ -138,7 +147,7 @@ void examine_Data(const char *func_name, int theLine, int what)
       {
          if (pc->orig_line == 6)
          {
-            if (pc->type == CT_NEWLINE)
+            if (chunk_is_token(pc, CT_NEWLINE))
             {
                LOG_FMT(LGUY, "(%zu)<NL> col=%zu\n\n", pc->orig_line, pc->orig_col);
             }
@@ -340,3 +349,4 @@ void dump_in(unsigned int type)
       exit(EX_SOFTWARE);
    }
 } // dump_in
+#endif /* DEVELOP_ONLY */
