@@ -683,6 +683,7 @@ EMSCRIPTEN_BINDINGS(MainModule)
       .value(STRINGIFY(UO_sp_before_unnamed_ptr_star), UO_sp_before_unnamed_ptr_star)
       .value(STRINGIFY(UO_sp_between_ptr_star), UO_sp_between_ptr_star)
       .value(STRINGIFY(UO_sp_after_ptr_star), UO_sp_after_ptr_star)
+      .value(STRINGIFY(UO_sp_after_ptr_block_caret), UO_sp_after_ptr_block_caret)
       .value(STRINGIFY(UO_sp_after_ptr_star_qualifier), UO_sp_after_ptr_star_qualifier)
       .value(STRINGIFY(UO_sp_after_ptr_star_func), UO_sp_after_ptr_star_func)
       .value(STRINGIFY(UO_sp_ptr_star_paren), UO_sp_ptr_star_paren)
@@ -721,7 +722,9 @@ EMSCRIPTEN_BINDINGS(MainModule)
       .value(STRINGIFY(UO_sp_after_semi_for_empty), UO_sp_after_semi_for_empty)
       .value(STRINGIFY(UO_sp_before_square), UO_sp_before_square)
       .value(STRINGIFY(UO_sp_before_squares), UO_sp_before_squares)
+      .value(STRINGIFY(UO_sp_cpp_before_struct_binding), UO_sp_cpp_before_struct_binding)
       .value(STRINGIFY(UO_sp_inside_square), UO_sp_inside_square)
+      .value(STRINGIFY(UO_sp_inside_square_oc_array), UO_sp_inside_square_oc_array)
       .value(STRINGIFY(UO_sp_after_comma), UO_sp_after_comma)
       .value(STRINGIFY(UO_sp_before_comma), UO_sp_before_comma)
       .value(STRINGIFY(UO_sp_after_mdatype_commas), UO_sp_after_mdatype_commas)
@@ -744,6 +747,7 @@ EMSCRIPTEN_BINDINGS(MainModule)
       .value(STRINGIFY(UO_sp_after_tag), UO_sp_after_tag)
       .value(STRINGIFY(UO_sp_inside_braces_enum), UO_sp_inside_braces_enum)
       .value(STRINGIFY(UO_sp_inside_braces_struct), UO_sp_inside_braces_struct)
+      .value(STRINGIFY(UO_sp_inside_braces_oc_dict), UO_sp_inside_braces_oc_dict)
       .value(STRINGIFY(UO_sp_after_type_brace_init_lst_open), UO_sp_after_type_brace_init_lst_open)
       .value(STRINGIFY(UO_sp_before_type_brace_init_lst_close), UO_sp_before_type_brace_init_lst_close)
       .value(STRINGIFY(UO_sp_inside_type_brace_init_lst), UO_sp_inside_type_brace_init_lst)
@@ -761,10 +765,13 @@ EMSCRIPTEN_BINDINGS(MainModule)
       .value(STRINGIFY(UO_sp_after_tparen_close), UO_sp_after_tparen_close)
       .value(STRINGIFY(UO_sp_square_fparen), UO_sp_square_fparen)
       .value(STRINGIFY(UO_sp_fparen_brace), UO_sp_fparen_brace)
+      .value(STRINGIFY(UO_sp_fparen_brace_initializer), UO_sp_fparen_brace_initializer)
       .value(STRINGIFY(UO_sp_fparen_dbrace), UO_sp_fparen_dbrace)
       .value(STRINGIFY(UO_sp_func_call_paren), UO_sp_func_call_paren)
       .value(STRINGIFY(UO_sp_func_call_paren_empty), UO_sp_func_call_paren_empty)
       .value(STRINGIFY(UO_sp_func_call_user_paren), UO_sp_func_call_user_paren)
+      .value(STRINGIFY(UO_sp_func_call_user_inside_fparen), UO_sp_func_call_user_inside_fparen)
+      .value(STRINGIFY(UO_sp_func_call_user_paren_paren), UO_sp_func_call_user_paren_paren)
       .value(STRINGIFY(UO_sp_func_class_paren), UO_sp_func_class_paren)
       .value(STRINGIFY(UO_sp_func_class_paren_empty), UO_sp_func_class_paren_empty)
       .value(STRINGIFY(UO_sp_return_paren), UO_sp_return_paren)
@@ -773,6 +780,7 @@ EMSCRIPTEN_BINDINGS(MainModule)
       .value(STRINGIFY(UO_sp_throw_paren), UO_sp_throw_paren)
       .value(STRINGIFY(UO_sp_after_throw), UO_sp_after_throw)
       .value(STRINGIFY(UO_sp_catch_paren), UO_sp_catch_paren)
+      .value(STRINGIFY(UO_sp_oc_catch_paren), UO_sp_oc_catch_paren)
       .value(STRINGIFY(UO_sp_version_paren), UO_sp_version_paren)
       .value(STRINGIFY(UO_sp_scope_paren), UO_sp_scope_paren)
       .value(STRINGIFY(UO_sp_super_paren), UO_sp_super_paren)
@@ -783,7 +791,9 @@ EMSCRIPTEN_BINDINGS(MainModule)
       .value(STRINGIFY(UO_sp_brace_else), UO_sp_brace_else)
       .value(STRINGIFY(UO_sp_brace_typedef), UO_sp_brace_typedef)
       .value(STRINGIFY(UO_sp_catch_brace), UO_sp_catch_brace)
+      .value(STRINGIFY(UO_sp_oc_catch_brace), UO_sp_oc_catch_brace)
       .value(STRINGIFY(UO_sp_brace_catch), UO_sp_brace_catch)
+      .value(STRINGIFY(UO_sp_oc_brace_catch), UO_sp_oc_brace_catch)
       .value(STRINGIFY(UO_sp_finally_brace), UO_sp_finally_brace)
       .value(STRINGIFY(UO_sp_brace_finally), UO_sp_brace_finally)
       .value(STRINGIFY(UO_sp_try_brace), UO_sp_try_brace)
@@ -817,6 +827,7 @@ EMSCRIPTEN_BINDINGS(MainModule)
       .value(STRINGIFY(UO_sp_after_oc_block_caret), UO_sp_after_oc_block_caret)
       .value(STRINGIFY(UO_sp_after_oc_msg_receiver), UO_sp_after_oc_msg_receiver)
       .value(STRINGIFY(UO_sp_after_oc_property), UO_sp_after_oc_property)
+      .value(STRINGIFY(UO_sp_after_oc_synchronized), UO_sp_after_oc_synchronized)
       .value(STRINGIFY(UO_sp_cond_colon), UO_sp_cond_colon)
       .value(STRINGIFY(UO_sp_cond_colon_before), UO_sp_cond_colon_before)
       .value(STRINGIFY(UO_sp_cond_colon_after), UO_sp_cond_colon_after)
@@ -843,9 +854,12 @@ EMSCRIPTEN_BINDINGS(MainModule)
       .value(STRINGIFY(UO_sp_num_before_tr_emb_cmt), UO_sp_num_before_tr_emb_cmt)
       .value(STRINGIFY(UO_sp_annotation_paren), UO_sp_annotation_paren)
       .value(STRINGIFY(UO_sp_skip_vbrace_tokens), UO_sp_skip_vbrace_tokens)
+      .value(STRINGIFY(UO_sp_after_noexcept), UO_sp_after_noexcept)
       .value(STRINGIFY(UO_force_tab_after_define), UO_force_tab_after_define)
       .value(STRINGIFY(UO_indent_columns), UO_indent_columns)
       .value(STRINGIFY(UO_indent_continue), UO_indent_continue)
+      .value(STRINGIFY(UO_indent_continue_class_head), UO_indent_continue_class_head)
+      .value(STRINGIFY(UO_indent_single_newlines), UO_indent_single_newlines)
       .value(STRINGIFY(UO_indent_param), UO_indent_param)
       .value(STRINGIFY(UO_indent_with_tabs), UO_indent_with_tabs)
       .value(STRINGIFY(UO_indent_cmt_with_tabs), UO_indent_cmt_with_tabs)
@@ -859,6 +873,7 @@ EMSCRIPTEN_BINDINGS(MainModule)
       .value(STRINGIFY(UO_indent_brace_parent), UO_indent_brace_parent)
       .value(STRINGIFY(UO_indent_paren_open_brace), UO_indent_paren_open_brace)
       .value(STRINGIFY(UO_indent_cs_delegate_brace), UO_indent_cs_delegate_brace)
+      .value(STRINGIFY(UO_indent_cs_delegate_body), UO_indent_cs_delegate_body)
       .value(STRINGIFY(UO_indent_namespace), UO_indent_namespace)
       .value(STRINGIFY(UO_indent_namespace_single_indent), UO_indent_namespace_single_indent)
       .value(STRINGIFY(UO_indent_namespace_level), UO_indent_namespace_level)
@@ -885,6 +900,7 @@ EMSCRIPTEN_BINDINGS(MainModule)
       .value(STRINGIFY(UO_indent_func_const), UO_indent_func_const)
       .value(STRINGIFY(UO_indent_func_throw), UO_indent_func_throw)
       .value(STRINGIFY(UO_indent_member), UO_indent_member)
+      .value(STRINGIFY(UO_indent_member_single), UO_indent_member_single)
       .value(STRINGIFY(UO_indent_sing_line_comments), UO_indent_sing_line_comments)
       .value(STRINGIFY(UO_indent_relative_single_line_comments), UO_indent_relative_single_line_comments)
       .value(STRINGIFY(UO_indent_switch_case), UO_indent_switch_case)
@@ -902,10 +918,13 @@ EMSCRIPTEN_BINDINGS(MainModule)
       .value(STRINGIFY(UO_indent_paren_after_func_call), UO_indent_paren_after_func_call)
       .value(STRINGIFY(UO_indent_comma_paren), UO_indent_comma_paren)
       .value(STRINGIFY(UO_indent_bool_paren), UO_indent_bool_paren)
+      .value(STRINGIFY(UO_indent_semicolon_for_paren), UO_indent_semicolon_for_paren)
       .value(STRINGIFY(UO_indent_first_bool_expr), UO_indent_first_bool_expr)
+      .value(STRINGIFY(UO_indent_first_for_expr), UO_indent_first_for_expr)
       .value(STRINGIFY(UO_indent_square_nl), UO_indent_square_nl)
       .value(STRINGIFY(UO_indent_preserve_sql), UO_indent_preserve_sql)
       .value(STRINGIFY(UO_indent_align_assign), UO_indent_align_assign)
+      .value(STRINGIFY(UO_indent_align_paren), UO_indent_align_paren)
       .value(STRINGIFY(UO_indent_oc_block), UO_indent_oc_block)
       .value(STRINGIFY(UO_indent_oc_block_msg), UO_indent_oc_block_msg)
       .value(STRINGIFY(UO_indent_oc_msg_colon), UO_indent_oc_msg_colon)
@@ -921,24 +940,31 @@ EMSCRIPTEN_BINDINGS(MainModule)
       .value(STRINGIFY(UO_indent_cpp_lambda_body), UO_indent_cpp_lambda_body)
       .value(STRINGIFY(UO_indent_using_block), UO_indent_using_block)
       .value(STRINGIFY(UO_indent_ternary_operator), UO_indent_ternary_operator)
+      .value(STRINGIFY(UO_indent_off_after_return_new), UO_indent_off_after_return_new)
+      .value(STRINGIFY(UO_indent_single_after_return), UO_indent_single_after_return)
       .value(STRINGIFY(UO_indent_ignore_asm_block), UO_indent_ignore_asm_block)
       .value(STRINGIFY(UO_nl_collapse_empty_body), UO_nl_collapse_empty_body)
       .value(STRINGIFY(UO_nl_assign_leave_one_liners), UO_nl_assign_leave_one_liners)
       .value(STRINGIFY(UO_nl_class_leave_one_liners), UO_nl_class_leave_one_liners)
       .value(STRINGIFY(UO_nl_enum_leave_one_liners), UO_nl_enum_leave_one_liners)
       .value(STRINGIFY(UO_nl_getset_leave_one_liners), UO_nl_getset_leave_one_liners)
+      .value(STRINGIFY(UO_nl_cs_property_leave_one_liners), UO_nl_cs_property_leave_one_liners)
       .value(STRINGIFY(UO_nl_func_leave_one_liners), UO_nl_func_leave_one_liners)
       .value(STRINGIFY(UO_nl_cpp_lambda_leave_one_liners), UO_nl_cpp_lambda_leave_one_liners)
       .value(STRINGIFY(UO_nl_if_leave_one_liners), UO_nl_if_leave_one_liners)
       .value(STRINGIFY(UO_nl_while_leave_one_liners), UO_nl_while_leave_one_liners)
+      .value(STRINGIFY(UO_nl_for_leave_one_liners), UO_nl_for_leave_one_liners)
       .value(STRINGIFY(UO_nl_oc_msg_leave_one_liner), UO_nl_oc_msg_leave_one_liner)
       .value(STRINGIFY(UO_nl_oc_block_brace), UO_nl_oc_block_brace)
+      .value(STRINGIFY(UO_nl_oc_interface_brace), UO_nl_oc_interface_brace)
+      .value(STRINGIFY(UO_nl_oc_implementation_brace), UO_nl_oc_implementation_brace)
       .value(STRINGIFY(UO_nl_start_of_file), UO_nl_start_of_file)
       .value(STRINGIFY(UO_nl_start_of_file_min), UO_nl_start_of_file_min)
       .value(STRINGIFY(UO_nl_end_of_file), UO_nl_end_of_file)
       .value(STRINGIFY(UO_nl_end_of_file_min), UO_nl_end_of_file_min)
       .value(STRINGIFY(UO_nl_assign_brace), UO_nl_assign_brace)
       .value(STRINGIFY(UO_nl_assign_square), UO_nl_assign_square)
+      .value(STRINGIFY(UO_nl_tsquare_brace), UO_nl_tsquare_brace)
       .value(STRINGIFY(UO_nl_after_square_assign), UO_nl_after_square_assign)
       .value(STRINGIFY(UO_nl_func_var_def_blk), UO_nl_func_var_def_blk)
       .value(STRINGIFY(UO_nl_typedef_blk_start), UO_nl_typedef_blk_start)
@@ -967,7 +993,9 @@ EMSCRIPTEN_BINDINGS(MainModule)
       .value(STRINGIFY(UO_nl_getset_brace), UO_nl_getset_brace)
       .value(STRINGIFY(UO_nl_for_brace), UO_nl_for_brace)
       .value(STRINGIFY(UO_nl_catch_brace), UO_nl_catch_brace)
+      .value(STRINGIFY(UO_nl_oc_catch_brace), UO_nl_oc_catch_brace)
       .value(STRINGIFY(UO_nl_brace_catch), UO_nl_brace_catch)
+      .value(STRINGIFY(UO_nl_oc_brace_catch), UO_nl_oc_brace_catch)
       .value(STRINGIFY(UO_nl_brace_square), UO_nl_brace_square)
       .value(STRINGIFY(UO_nl_brace_fparen), UO_nl_brace_fparen)
       .value(STRINGIFY(UO_nl_while_brace), UO_nl_while_brace)
@@ -1042,6 +1070,7 @@ EMSCRIPTEN_BINDINGS(MainModule)
       .value(STRINGIFY(UO_nl_after_vbrace_close), UO_nl_after_vbrace_close)
       .value(STRINGIFY(UO_nl_brace_struct_var), UO_nl_brace_struct_var)
       .value(STRINGIFY(UO_nl_define_macro), UO_nl_define_macro)
+      .value(STRINGIFY(UO_nl_squeeze_paren_close), UO_nl_squeeze_paren_close)
       .value(STRINGIFY(UO_nl_squeeze_ifdef), UO_nl_squeeze_ifdef)
       .value(STRINGIFY(UO_nl_squeeze_ifdef_top_level), UO_nl_squeeze_ifdef_top_level)
       .value(STRINGIFY(UO_nl_before_if), UO_nl_before_if)
@@ -1062,9 +1091,11 @@ EMSCRIPTEN_BINDINGS(MainModule)
       .value(STRINGIFY(UO_nl_before_func_class_proto), UO_nl_before_func_class_proto)
       .value(STRINGIFY(UO_nl_class_colon), UO_nl_class_colon)
       .value(STRINGIFY(UO_nl_constr_colon), UO_nl_constr_colon)
+      .value(STRINGIFY(UO_nl_namespace_two_to_one_liner), UO_nl_namespace_two_to_one_liner)
       .value(STRINGIFY(UO_nl_create_if_one_liner), UO_nl_create_if_one_liner)
       .value(STRINGIFY(UO_nl_create_for_one_liner), UO_nl_create_for_one_liner)
       .value(STRINGIFY(UO_nl_create_while_one_liner), UO_nl_create_while_one_liner)
+      .value(STRINGIFY(UO_nl_create_func_def_one_liner), UO_nl_create_func_def_one_liner)
       .value(STRINGIFY(UO_nl_split_if_one_liner), UO_nl_split_if_one_liner)
       .value(STRINGIFY(UO_nl_split_for_one_liner), UO_nl_split_for_one_liner)
       .value(STRINGIFY(UO_nl_split_while_one_liner), UO_nl_split_while_one_liner)
@@ -1126,6 +1157,8 @@ EMSCRIPTEN_BINDINGS(MainModule)
       .value(STRINGIFY(UO_align_func_params_thresh), UO_align_func_params_thresh)
       .value(STRINGIFY(UO_align_func_params_gap), UO_align_func_params_gap)
       .value(STRINGIFY(UO_align_same_func_call_params), UO_align_same_func_call_params)
+      .value(STRINGIFY(UO_align_same_func_call_params_span), UO_align_same_func_call_params_span)
+      .value(STRINGIFY(UO_align_same_func_call_params_thresh), UO_align_same_func_call_params_thresh)
       .value(STRINGIFY(UO_align_var_def_span), UO_align_var_def_span)
       .value(STRINGIFY(UO_align_var_def_star_style), UO_align_var_def_star_style)
       .value(STRINGIFY(UO_align_var_def_amp_style), UO_align_var_def_amp_style)
@@ -1250,6 +1283,7 @@ EMSCRIPTEN_BINDINGS(MainModule)
       .value(STRINGIFY(UO_include_category_2), UO_include_category_2)
       .value(STRINGIFY(UO_use_indent_func_call_param), UO_use_indent_func_call_param)
       .value(STRINGIFY(UO_use_indent_continue_only_once), UO_use_indent_continue_only_once)
+      .value(STRINGIFY(UO_indent_cpp_lambda_only_once), UO_indent_cpp_lambda_only_once)
       .value(STRINGIFY(UO_use_options_overriding_for_qt_macros), UO_use_options_overriding_for_qt_macros)
       .value(STRINGIFY(UO_warn_level_tabs_found_in_verbatim_string_literals), UO_warn_level_tabs_found_in_verbatim_string_literals)
       .value(STRINGIFY(UO_option_count), UO_option_count);
@@ -1429,6 +1463,7 @@ EMSCRIPTEN_BINDINGS(MainModule)
       .value(STRINGIFY(CT_TYPE_CAST), CT_TYPE_CAST)
       .value(STRINGIFY(CT_TYPENAME), CT_TYPENAME)
       .value(STRINGIFY(CT_TEMPLATE), CT_TEMPLATE)
+      .value(STRINGIFY(CT_WHERE_SPEC), CT_WHERE_SPEC)
       .value(STRINGIFY(CT_ASSIGN), CT_ASSIGN)
       .value(STRINGIFY(CT_ASSIGN_NL), CT_ASSIGN_NL)
       .value(STRINGIFY(CT_SASSIGN), CT_SASSIGN)
@@ -1474,12 +1509,16 @@ EMSCRIPTEN_BINDINGS(MainModule)
       .value(STRINGIFY(CT_CONSTR_COLON), CT_CONSTR_COLON)
       .value(STRINGIFY(CT_D_ARRAY_COLON), CT_D_ARRAY_COLON)
       .value(STRINGIFY(CT_COND_COLON), CT_COND_COLON)
+      .value(STRINGIFY(CT_WHERE_COLON), CT_WHERE_COLON)
       .value(STRINGIFY(CT_QUESTION), CT_QUESTION)
       .value(STRINGIFY(CT_COMMA), CT_COMMA)
       .value(STRINGIFY(CT_ASM), CT_ASM)
       .value(STRINGIFY(CT_ATTRIBUTE), CT_ATTRIBUTE)
+      .value(STRINGIFY(CT_OC_AVAILABLE), CT_OC_AVAILABLE)
+      .value(STRINGIFY(CT_OC_AVAILABLE_VALUE), CT_OC_AVAILABLE_VALUE)
       .value(STRINGIFY(CT_CATCH), CT_CATCH)
       .value(STRINGIFY(CT_WHEN), CT_WHEN)
+      .value(STRINGIFY(CT_WHERE), CT_WHERE)
       .value(STRINGIFY(CT_CLASS), CT_CLASS)
       .value(STRINGIFY(CT_DELETE), CT_DELETE)
       .value(STRINGIFY(CT_EXPORT), CT_EXPORT)
@@ -1497,6 +1536,7 @@ EMSCRIPTEN_BINDINGS(MainModule)
       .value(STRINGIFY(CT_BRACED_INIT_LIST), CT_BRACED_INIT_LIST)
       .value(STRINGIFY(CT_USING), CT_USING)
       .value(STRINGIFY(CT_USING_STMT), CT_USING_STMT)
+      .value(STRINGIFY(CT_USING_ALIAS), CT_USING_ALIAS)
       .value(STRINGIFY(CT_D_WITH), CT_D_WITH)
       .value(STRINGIFY(CT_D_MODULE), CT_D_MODULE)
       .value(STRINGIFY(CT_SUPER), CT_SUPER)
@@ -1508,6 +1548,7 @@ EMSCRIPTEN_BINDINGS(MainModule)
       .value(STRINGIFY(CT_UNITTEST), CT_UNITTEST)
       .value(STRINGIFY(CT_UNSAFE), CT_UNSAFE)
       .value(STRINGIFY(CT_FINALLY), CT_FINALLY)
+      .value(STRINGIFY(CT_FIXED), CT_FIXED)
       .value(STRINGIFY(CT_IMPORT), CT_IMPORT)
       .value(STRINGIFY(CT_D_SCOPE), CT_D_SCOPE)
       .value(STRINGIFY(CT_D_SCOPE_IF), CT_D_SCOPE_IF)
@@ -1544,6 +1585,7 @@ EMSCRIPTEN_BINDINGS(MainModule)
       .value(STRINGIFY(CT_FUNC_TYPE), CT_FUNC_TYPE)
       .value(STRINGIFY(CT_FUNC_VAR), CT_FUNC_VAR)
       .value(STRINGIFY(CT_FUNC_PROTO), CT_FUNC_PROTO)
+      .value(STRINGIFY(CT_FUNC_START), CT_FUNC_START)
       .value(STRINGIFY(CT_FUNC_CLASS_DEF), CT_FUNC_CLASS_DEF)
       .value(STRINGIFY(CT_FUNC_CLASS_PROTO), CT_FUNC_CLASS_PROTO)
       .value(STRINGIFY(CT_FUNC_CTOR_VAR), CT_FUNC_CTOR_VAR)
@@ -1553,6 +1595,7 @@ EMSCRIPTEN_BINDINGS(MainModule)
       .value(STRINGIFY(CT_MACRO), CT_MACRO)
       .value(STRINGIFY(CT_QUALIFIER), CT_QUALIFIER)
       .value(STRINGIFY(CT_EXTERN), CT_EXTERN)
+      .value(STRINGIFY(CT_DECLSPEC), CT_DECLSPEC)
       .value(STRINGIFY(CT_ALIGN), CT_ALIGN)
       .value(STRINGIFY(CT_TYPE), CT_TYPE)
       .value(STRINGIFY(CT_PTR_TYPE), CT_PTR_TYPE)
@@ -1642,12 +1685,15 @@ EMSCRIPTEN_BINDINGS(MainModule)
       .value(STRINGIFY(CT_SQL_BEGIN), CT_SQL_BEGIN)
       .value(STRINGIFY(CT_SQL_END), CT_SQL_END)
       .value(STRINGIFY(CT_SQL_WORD), CT_SQL_WORD)
+      .value(STRINGIFY(CT_SQL_ASSIGN), CT_SQL_ASSIGN)
       .value(STRINGIFY(CT_CONSTRUCT), CT_CONSTRUCT)
       .value(STRINGIFY(CT_LAMBDA), CT_LAMBDA)
       .value(STRINGIFY(CT_ASSERT), CT_ASSERT)
       .value(STRINGIFY(CT_ANNOTATION), CT_ANNOTATION)
       .value(STRINGIFY(CT_FOR_COLON), CT_FOR_COLON)
       .value(STRINGIFY(CT_DOUBLE_BRACE), CT_DOUBLE_BRACE)
+      .value(STRINGIFY(CT_CNG_HASINC), CT_CNG_HASINC)
+      .value(STRINGIFY(CT_CNG_HASINCN), CT_CNG_HASINCN)
       .value(STRINGIFY(CT_Q_EMIT), CT_Q_EMIT)
       .value(STRINGIFY(CT_Q_FOREACH), CT_Q_FOREACH)
       .value(STRINGIFY(CT_Q_FOREVER), CT_Q_FOREVER)
@@ -1674,6 +1720,7 @@ EMSCRIPTEN_BINDINGS(MainModule)
       .value(STRINGIFY(LANG_ECMA), LANG_ECMA)
       .value(STRINGIFY(LANG_ALLC), LANG_ALLC)
       .value(STRINGIFY(LANG_ALL), LANG_ALL)
+      .value(STRINGIFY(FLAG_HDR), FLAG_HDR)
       .value(STRINGIFY(FLAG_DIG), FLAG_DIG)
       .value(STRINGIFY(FLAG_PP), FLAG_PP);
 
