@@ -14,6 +14,7 @@
 
 #include "base_types.h"
 #include "options.h"
+#include "option_enum.h"
 #include "token_enum.h"    // c_token_t
 #include "log_levels.h"
 #include "logger.h"
@@ -24,6 +25,8 @@
 #include <utime.h>
 #endif
 
+class ParseFrame;
+
 
 /**
  * abbreviations used:
@@ -40,8 +43,11 @@
 //! returns type (with removed reference) of a variable
 #define noref_decl_t(X)              std::remove_reference<decltype((X))>::type
 
+//! returns type (with removed const and reference) of a variable
+#define nocref_decl_t(X)             std::remove_const<noref_decl_t((X))>::type
+
 //! static casts Y to the type (with removed reference) of X
-#define s_cast_noref_decl_t(X, Y)    static_cast<noref_decl_t(X)>(Y)
+#define s_cast_noref_decl_t(X, Y)    static_cast<nocref_decl_t(X)>(Y)
 
 //! performs abs on Y after static casting it to the type (with removed reference) of X
 #define cast_abs(X, Y)               s_cast_noref_decl_t(X, abs(Y))
@@ -59,12 +65,12 @@
 enum class brace_stage_e : unsigned int
 {
    NONE,
-   PAREN1,      //! if/for/switch/while/synchronized
-   OP_PAREN1,   //! optional paren: catch () {
+   PAREN1,      //! expected paren after if/catch (C++)/for/switch/synchronized/while
+   OP_PAREN1,   //! optional paren after catch (C#)
    WOD_PAREN,   //! while of do parens
    WOD_SEMI,    //! semicolon after while of do
    BRACE_DO,    //! do
-   BRACE2,      //! if/else/for/switch/while
+   BRACE2,      //! if/catch/else/finally/for/switch/synchronized/while
    ELSE,        //! expecting 'else' after 'if'
    ELSEIF,      //! expecting 'if' after 'else'
    WHILE,       //! expecting 'while' after 'do'
@@ -72,6 +78,7 @@ enum class brace_stage_e : unsigned int
    CATCH_WHEN,  //! optional 'when' after 'catch'
 };
 
+
 enum class char_encoding_e : unsigned int
 {
    e_ASCII,     //! 0-127
@@ -97,50 +104,6 @@ struct indent_ptr_t
 };
 
 
-//! Structure for counting nested level
-struct paren_stack_entry_t
-{
-   c_token_t     type;         //! the type that opened the entry
-   size_t        level;        //! Level of opening type
-   size_t        open_line;    //! line that open symbol is on
-   chunk_t       *pc;          //! Chunk that opened the level
-   int           brace_indent; //! indent for braces - may not relate to indent
-   size_t        indent;       //! indent level (depends on use)
-   size_t        indent_tmp;   //! temporary indent level (depends on use)
-   size_t        indent_tab;   //! the 'tab' indent (always <= real column)
-   bool          indent_cont;  //! indent_continue was applied
-   int           ref;
-   c_token_t     parent;       //! if, for, function, etc
-   brace_stage_e stage;
-   bool          in_preproc;   //! whether this was created in a preprocessor
-   size_t        ns_cnt;       //! Number of consecutive namespace levels
-   bool          non_vardef;   //! Hit a non-vardef line
-   indent_ptr_t  ip;
-};
-
-// TODO: put this on a linked list
-struct parse_frame_t
-{
-   int                 ref_no;
-   int                 level;           //! level of parens/square/angle/brace
-   int                 brace_level;     //! level of brace/vbrace
-   int                 pp_level;        //! level of preproc #if stuff
-
-   int                 sparen_count;
-
-#define PSE_SIZE    128
-   paren_stack_entry_t pse[PSE_SIZE];
-   size_t              pse_tos;
-   int                 paren_count;
-
-   c_token_t           in_ifdef;
-   int                 stmt_count;
-   int                 expr_count;
-
-   bool                maybe_decl;
-   bool                maybe_cast;
-};
-
 #define PCF_BIT(b)    (1ULL << b)
 
 // Copy flags are in the lower 16 bits
@@ -160,6 +123,7 @@ struct parse_frame_t
 #define PCF_IN_NAMESPACE       PCF_BIT(12)
 #define PCF_IN_FOR             PCF_BIT(13)
 #define PCF_IN_OC_MSG          PCF_BIT(14)
+#define PCF_IN_WHERE_SPEC      PCF_BIT(15)  /* inside C# 'where' constraint clause on class or function def */
 
 // Non-Copy flags are in the upper 48 bits
 #define PCF_FORCE_SPACE        PCF_BIT(16)  //! must have a space after this token
@@ -282,6 +246,7 @@ struct chunk_t
       column        = 0;
       column_indent = 0;
       nl_count      = 0;
+      nl_column     = 0;
       level         = 0;
       brace_level   = 0;
       pp_level      = 0;
@@ -319,6 +284,7 @@ struct chunk_t
                                    * column, which may be less than the real
                                    * column used to indent with tabs          */
    size_t       nl_count;         //! number of newlines in CT_NEWLINE
+   size_t       nl_column;        //! column of the subsequent newline entries(all of them should have the same column)
    size_t       level;            //! nest level in {, (, or [
    size_t       brace_level;      //! nest level in braces only
    size_t       pp_level;         //! nest level in preprocessor
@@ -344,6 +310,7 @@ enum lang_flag_e
                             *  LANG_JAVA | LANG_OC  | LANG_VALA | LANG_ECMA   */
    LANG_ALL  = 0x0fff,     //! applies to all languages
 
+   FLAG_HDR  = 0x2000,     /*<< Header file for C family languages */
    FLAG_DIG  = 0x4000,     //! digraph/trigraph
    FLAG_PP   = 0x8000,     //! only appears in a preprocessor
 };
@@ -416,74 +383,69 @@ enum class unc_stage_e : unsigned int
 
 struct cp_data_t
 {
-   std::deque<UINT8> *bout;
-   FILE              *fout;
-   int               last_char;
-   bool              do_check;
-   unc_stage_e       unc_stage;
-   int               check_fail_cnt; //! total failure count
-   bool              if_changed;
+   std::deque<UINT8>       *bout;
+   FILE                    *fout;
+   int                     last_char;
+   bool                    do_check;
+   unc_stage_e             unc_stage;
+   int                     check_fail_cnt; //! total failure count
+   bool                    if_changed;
 
-   UINT32            error_count;   //! counts how many errors occurred so far
-   std::string       filename;
+   UINT32                  error_count; //! counts how many errors occurred so far
+   std::string             filename;
 
-   file_mem          file_hdr;      // for cmt_insert_file_header
-   file_mem          file_ftr;      // for cmt_insert_file_footer
-   file_mem          func_hdr;      // for cmt_insert_func_header
-   file_mem          oc_msg_hdr;    // for cmt_insert_oc_msg_header
-   file_mem          class_hdr;     // for cmt_insert_class_header
+   file_mem                file_hdr;    // for cmt_insert_file_header
+   file_mem                file_ftr;    // for cmt_insert_file_footer
+   file_mem                func_hdr;    // for cmt_insert_func_header
+   file_mem                oc_msg_hdr;  // for cmt_insert_oc_msg_header
+   file_mem                class_hdr;   // for cmt_insert_class_header
 
-   size_t            lang_flags;    //! defines the language of the source input
-   bool              lang_forced;   //! overwrites automatic language detection
+   size_t                  lang_flags;  //! defines the language of the source input
+   bool                    lang_forced; //! overwrites automatic language detection
 
-   bool              unc_off;
-   bool              unc_off_used;  //! to check if "unc_off" is used
-   UINT32            line_number;
-   size_t            column;        //! column for parsing
-   UINT16            spaces;        //! space count on output
+   bool                    unc_off;
+   bool                    unc_off_used; //! to check if "unc_off" is used
+   UINT32                  line_number;
+   size_t                  column;       //! column for parsing
+   UINT16                  spaces;       //! space count on output
 
-   int               ifdef_over_whole_file;
+   int                     ifdef_over_whole_file;
 
-   bool              frag;          //! activates code fragment option
-   UINT32            frag_cols;
+   bool                    frag;    //! activates code fragment option
+   UINT32                  frag_cols;
 
    // stuff to auto-detect line endings
-   UINT32            le_counts[LE_AUTO];
-   unc_text          newline;
+   UINT32                  le_counts[uncrustify::line_end_styles];
+   unc_text                newline;
 
-   bool              consumed;
+   bool                    consumed;
 
-   int               did_newline;   //! flag indicates if a newline was added or converted
-   c_token_t         in_preproc;
-   int               preproc_ncnl_count;
-   bool              output_trailspace;
-   bool              output_tab_as_space;
+   int                     did_newline; //! flag indicates if a newline was added or converted
+   c_token_t               in_preproc;
+   int                     preproc_ncnl_count;
+   bool                    output_trailspace;
+   bool                    output_tab_as_space;
 
-   bool              bom;
-   char_encoding_e   enc;
+   bool                    bom;
+   char_encoding_e         enc;
 
    // bumped up when a line is split or indented
-   int               changes;
-   int               pass_count; //! indicates how often the chunk list shall be processed
+   int                     changes;
+   int                     pass_count; //! indicates how often the chunk list shall be processed
 
 #define AL_SIZE    8000
-   align_t           al[AL_SIZE];
-   size_t            al_cnt;
-   bool              al_c99_array;
+   align_t                 al[AL_SIZE];
+   size_t                  al_cnt;
+   bool                    al_c99_array;
 
-   bool              warned_unable_string_replace_tab_chars;
+   bool                    warned_unable_string_replace_tab_chars;
 
-   op_val_t          settings[UO_option_count]; //! array with all uncrustify option settings
+   std::vector<ParseFrame> frames;
+//   size_t                  frame_count;
+   int                     pp_level; // TODO: can this ever be -1?
 
-#define FRAME_SIZE    16
-   parse_frame_t     frames[FRAME_SIZE];
-   size_t            frame_count;
-   int               pp_level; // TODO: can this ever be -1?
-
-   // the default values for settings
-   op_val_t          defaults[UO_option_count];
-   const char        *phase_name;
-   const char        *dumped_file;
+   const char              *phase_name;
+   const char              *dumped_file;
 };
 
 extern cp_data_t cpd;  // TODO: can we avoid this external variable?
