@@ -16,6 +16,8 @@
 
 #include <cstdlib>
 
+using namespace uncrustify;
+
 
 /**
  * abbreviations used:
@@ -36,7 +38,7 @@ struct token_pri
 };
 
 
-static_inline bool is_past_width(chunk_t *pc);
+static inline bool is_past_width(chunk_t *pc);
 
 
 //! Split right after the chunk
@@ -125,24 +127,24 @@ static void split_fcn_params_full(chunk_t *start);
 static void split_for_stmt(chunk_t *start);
 
 
-static_inline bool is_past_width(chunk_t *pc)
+static inline bool is_past_width(chunk_t *pc)
 {
    // allow char to sit at last column by subtracting 1
-   return((pc->column + pc->len() - 1) > cpd.settings[UO_code_width].u);
+   return((pc->column + pc->len() - 1) > options::code_width());
 }
 
 
 static void split_before_chunk(chunk_t *pc)
 {
    LOG_FUNC_ENTRY();
-   LOG_FMT(LSPLIT, "  %s: %s\n", __func__, pc->text());
+   LOG_FMT(LSPLIT, "%s(%d): text() '%s'\n", __func__, __LINE__, pc->text());
 
    if (!chunk_is_newline(pc) && !chunk_is_newline(chunk_get_prev(pc)))
    {
       newline_add_before(pc);
       // reindent needs to include the indent_continue value and was off by one
-      reindent_line(pc, pc->brace_level * cpd.settings[UO_indent_columns].u +
-                    abs(cpd.settings[UO_indent_continue].n) + 1);
+      reindent_line(pc, pc->brace_level * options::indent_columns() +
+                    abs(options::indent_continue()) + 1);
       cpd.changes++;
    }
 }
@@ -157,11 +159,11 @@ void do_code_width(void)
    {
       if (  !chunk_is_newline(pc)
          && !chunk_is_comment(pc)
-         && pc->type != CT_SPACE
+         && chunk_is_not_token(pc, CT_SPACE)
          && is_past_width(pc))
       {
-         if (  pc->type == CT_VBRACE_CLOSE // don't break if a vbrace close
-            && chunk_is_last_on_line(*pc)) // is the last chunk on its line
+         if (  chunk_is_token(pc, CT_VBRACE_CLOSE) // don't break if a vbrace close
+            && chunk_is_last_on_line(*pc))         // is the last chunk on its line
          {
             continue;
          }
@@ -227,7 +229,7 @@ static void try_split_here(cw_entry &ent, chunk_t *pc)
 
    LOG_FMT(LSPLIT, "%s(%d): at %s, orig_col=%zu\n", __func__, __LINE__, pc->text(), pc->orig_col);
    size_t pc_pri = get_split_pri(pc->type);
-   LOG_FMT(LSPLIT, "%s(%d): pc_pri=%zu\n", __func__, __LINE__, pc_pri);
+   LOG_FMT(LSPLIT, "%s(%d): pc_pri is %zu\n", __func__, __LINE__, pc_pri);
    if (pc_pri == 0)
    {
       LOG_FMT(LSPLIT, "%s(%d): pc_pri is 0, return\n", __func__, __LINE__);
@@ -238,11 +240,11 @@ static void try_split_here(cw_entry &ent, chunk_t *pc)
    // Can't split after a newline
    chunk_t *prev = chunk_get_prev(pc);
    if (  prev == nullptr
-      || (chunk_is_newline(prev) && pc->type != CT_STRING))
+      || (chunk_is_newline(prev) && chunk_is_not_token(pc, CT_STRING)))
    {
       if (prev != nullptr)
       {
-         LOG_FMT(LSPLIT, "%s(%d): Can't split after a newline, orig_line=%zu, return\n",
+         LOG_FMT(LSPLIT, "%s(%d): Can't split after a newline, orig_line is %zu, return\n",
                  __func__, __LINE__, prev->orig_line);
       }
       return;
@@ -250,10 +252,10 @@ static void try_split_here(cw_entry &ent, chunk_t *pc)
 
    LOG_FMT(LSPLIT, "%s(%d):\n", __func__, __LINE__);
    // Can't split a function without arguments
-   if (pc->type == CT_FPAREN_OPEN)
+   if (chunk_is_token(pc, CT_FPAREN_OPEN))
    {
       chunk_t *next = chunk_get_next(pc);
-      if (next->type == CT_FPAREN_CLOSE)
+      if (chunk_is_token(next, CT_FPAREN_CLOSE))
       {
          LOG_FMT(LSPLIT, "%s(%d): Can't split a function without arguments, return\n", __func__, __LINE__);
          return;
@@ -262,10 +264,10 @@ static void try_split_here(cw_entry &ent, chunk_t *pc)
 
    LOG_FMT(LSPLIT, "%s(%d):\n", __func__, __LINE__);
    // Only split concatenated strings
-   if (pc->type == CT_STRING)
+   if (chunk_is_token(pc, CT_STRING))
    {
       chunk_t *next = chunk_get_next(pc);
-      if (next->type != CT_STRING)
+      if (chunk_is_not_token(next, CT_STRING))
       {
          LOG_FMT(LSPLIT, "%s(%d): Only split concatenated strings, return\n", __func__, __LINE__);
          return;
@@ -274,7 +276,7 @@ static void try_split_here(cw_entry &ent, chunk_t *pc)
 
    LOG_FMT(LSPLIT, "%s(%d):\n", __func__, __LINE__);
    // keep common groupings unless ls_code_width
-   if (!cpd.settings[UO_ls_code_width].b && pc_pri >= 20)
+   if (!options::ls_code_width() && pc_pri >= 20)
    {
       LOG_FMT(LSPLIT, "%s(%d): keep common groupings unless ls_code_width, return\n", __func__, __LINE__);
       return;
@@ -285,7 +287,7 @@ static void try_split_here(cw_entry &ent, chunk_t *pc)
    if (pc_pri == 25)
    {
       chunk_t *next = chunk_get_next(pc);
-      if (next->type != CT_WORD && (get_split_pri(next->type) != 25))
+      if (chunk_is_not_token(next, CT_WORD) && (get_split_pri(next->type) != 25))
       {
          LOG_FMT(LSPLIT, "%s(%d): don't break after last term of a qualified type, return\n", __func__, __LINE__);
          return;
@@ -343,7 +345,7 @@ static bool split_line(chunk_t *start)
    }
 
    LOG_FMT(LSPLIT, "%s(%d):\n", __func__, __LINE__);
-   if (cpd.settings[UO_ls_code_width].b)
+   if (options::ls_code_width())
    {
    }
    // Check to see if we are in a for statement
@@ -369,7 +371,7 @@ static bool split_line(chunk_t *start)
    {
       LOG_FMT(LSPLIT, " ** FUNC SPLIT **\n");
 
-      if (cpd.settings[UO_ls_func_split_full].b)
+      if (options::ls_func_split_full())
       {
          split_fcn_params_full(start);
          if (!is_past_width(start))
@@ -401,13 +403,13 @@ static bool split_line(chunk_t *start)
 
    while (((pc = chunk_get_prev(pc)) != nullptr) && !chunk_is_newline(pc))
    {
-      LOG_FMT(LSPLIT, "%s(%d): at %s, orig_line=%zu, orig_col=%zu\n",
+      LOG_FMT(LSPLIT, "%s(%d): at %s, orig_line is %zu, orig_col is %zu\n",
               __func__, __LINE__, pc->text(), pc->orig_line, pc->orig_col);
-      if (pc->type != CT_SPACE)
+      if (chunk_is_not_token(pc, CT_SPACE))
       {
          try_split_here(ent, pc);
          // break at maximum line length
-         if (ent.pc != nullptr && (cpd.settings[UO_ls_code_width].b))
+         if (ent.pc != nullptr && (options::ls_code_width()))
          {
             break;
          }
@@ -416,14 +418,14 @@ static bool split_line(chunk_t *start)
 
    if (ent.pc == nullptr)
    {
-      LOG_FMT(LSPLIT, "\n%s(%d):    TRY_SPLIT yielded NO SOLUTION for orig_line %zu at %s [%s]\n",
+      LOG_FMT(LSPLIT, "\n%s(%d):    TRY_SPLIT yielded NO SOLUTION for orig_line %zu at '%s' [%s]\n",
               __func__, __LINE__, start->orig_line, start->text(), get_token_name(start->type));
    }
    else
    {
       LOG_FMT(LSPLIT, "\n%s(%d):    TRY_SPLIT yielded '%s' [%s] on orig_line %zu\n",
               __func__, __LINE__, ent.pc->text(), get_token_name(ent.pc->type), ent.pc->orig_line);
-      LOG_FMT(LSPLIT, "%s(%d): ent at %s, orig_col=%zu\n",
+      LOG_FMT(LSPLIT, "%s(%d): ent at '%s', orig_col is %zu\n",
               __func__, __LINE__, ent.pc->text(), ent.pc->orig_col);
    }
 
@@ -436,16 +438,16 @@ static bool split_line(chunk_t *start)
    {
       if (  (  (  chunk_is_token(ent.pc, CT_ARITH)
                || chunk_is_token(ent.pc, CT_CARET))
-            && (cpd.settings[UO_pos_arith].tp & TP_LEAD))
+            && (options::pos_arith() & TP_LEAD))
          || (  chunk_is_token(ent.pc, CT_ASSIGN)
-            && (cpd.settings[UO_pos_assign].tp & TP_LEAD))
+            && (options::pos_assign() & TP_LEAD))
          || (  chunk_is_token(ent.pc, CT_COMPARE)
-            && (cpd.settings[UO_pos_compare].tp & TP_LEAD))
+            && (options::pos_compare() & TP_LEAD))
          || (  (  chunk_is_token(ent.pc, CT_COND_COLON)
                || chunk_is_token(ent.pc, CT_QUESTION))
-            && (cpd.settings[UO_pos_conditional].tp & TP_LEAD))
+            && (options::pos_conditional() & TP_LEAD))
          || (  chunk_is_token(ent.pc, CT_BOOL)
-            && (cpd.settings[UO_pos_bool].tp & TP_LEAD)))
+            && (options::pos_bool() & TP_LEAD)))
       {
          pc = ent.pc;
       }
@@ -453,24 +455,25 @@ static bool split_line(chunk_t *start)
       {
          pc = chunk_get_next(ent.pc);
       }
-      LOG_FMT(LSPLIT, "%s(%d): at %s, col=%zu\n", __func__, __LINE__, pc->text(), pc->orig_col);
+      LOG_FMT(LSPLIT, "%s(%d): at '%s', orig_col is %zu\n",
+              __func__, __LINE__, pc->text(), pc->orig_col);
    }
 
    if (pc == nullptr)
    {
       pc = start;
       // Don't break before a close, comma, or colon
-      if (  start->type == CT_PAREN_CLOSE
-         || start->type == CT_PAREN_OPEN
-         || start->type == CT_FPAREN_CLOSE
-         || start->type == CT_FPAREN_OPEN
-         || start->type == CT_SPAREN_CLOSE
-         || start->type == CT_SPAREN_OPEN
-         || start->type == CT_ANGLE_CLOSE
-         || start->type == CT_BRACE_CLOSE
-         || start->type == CT_COMMA
-         || start->type == CT_SEMICOLON
-         || start->type == CT_VSEMICOLON
+      if (  chunk_is_token(start, CT_PAREN_CLOSE)
+         || chunk_is_token(start, CT_PAREN_OPEN)
+         || chunk_is_token(start, CT_FPAREN_CLOSE)
+         || chunk_is_token(start, CT_FPAREN_OPEN)
+         || chunk_is_token(start, CT_SPAREN_CLOSE)
+         || chunk_is_token(start, CT_SPAREN_OPEN)
+         || chunk_is_token(start, CT_ANGLE_CLOSE)
+         || chunk_is_token(start, CT_BRACE_CLOSE)
+         || chunk_is_token(start, CT_COMMA)
+         || chunk_is_token(start, CT_SEMICOLON)
+         || chunk_is_token(start, CT_VSEMICOLON)
          || start->len() == 0)
       {
          LOG_FMT(LSPLIT, " ** NO GO **\n");
@@ -491,7 +494,7 @@ static bool split_line(chunk_t *start)
       //LOG_FMT(LSPLIT, " '%.*s' [%s], started on token '%.*s' [%s]\n",
       //        plen, pc->text(), get_token_name(pc->type),
       //        slen, start->text(), get_token_name(start->type));
-      LOG_FMT(LSPLIT, "  %s(%d): %s [%s], started on token '%s' [%s]\n",
+      LOG_FMT(LSPLIT, "%s(%d): text() '%s', type %s, started on token '%s', type %s\n",
               __func__, __LINE__, pc->text(), get_token_name(pc->type),
               start->text(), get_token_name(start->type));
 
@@ -513,7 +516,7 @@ static void split_for_stmt(chunk_t *start)
 {
    LOG_FUNC_ENTRY();
    // how many semicolons (1 or 2) do we need to find
-   size_t  max_cnt     = cpd.settings[UO_ls_for_split_full].b ? 2 : 1;
+   size_t  max_cnt     = options::ls_for_split_full() ? 2 : 1;
    chunk_t *open_paren = nullptr;
    size_t  nl_cnt      = 0;
 
@@ -524,7 +527,7 @@ static void split_for_stmt(chunk_t *start)
    chunk_t *pc = start;
    while ((pc = chunk_get_prev(pc)) != nullptr)
    {
-      if (pc->type == CT_SPAREN_OPEN)
+      if (chunk_is_token(pc, CT_SPAREN_OPEN))
       {
          open_paren = pc;
          break;
@@ -544,7 +547,7 @@ static void split_for_stmt(chunk_t *start)
    int     count = 0;
    chunk_t *st[2];
    pc = start;
-   if (pc->type == CT_SEMICOLON && pc->parent_type == CT_FOR)
+   if (chunk_is_token(pc, CT_SEMICOLON) && pc->parent_type == CT_FOR)
    {
       st[count++] = pc;
    }
@@ -554,7 +557,7 @@ static void split_for_stmt(chunk_t *start)
          && ((pc = chunk_get_prev(pc)) != nullptr)
          && (pc->flags & PCF_IN_SPAREN))
    {
-      if (pc->type == CT_SEMICOLON && pc->parent_type == CT_FOR)
+      if (chunk_is_token(pc, CT_SEMICOLON) && pc->parent_type == CT_FOR)
       {
          st[count++] = pc;
       }
@@ -566,7 +569,7 @@ static void split_for_stmt(chunk_t *start)
          && ((pc = chunk_get_next(pc)) != nullptr)
          && (pc->flags & PCF_IN_SPAREN))
    {
-      if (pc->type == CT_SEMICOLON && pc->parent_type == CT_FOR)
+      if (chunk_is_token(pc, CT_SEMICOLON) && pc->parent_type == CT_FOR)
       {
          st[count++] = pc;
       }
@@ -575,7 +578,7 @@ static void split_for_stmt(chunk_t *start)
    while (--count >= 0)
    {
       // TODO: st[0] may be uninitialized here
-      LOG_FMT(LSPLIT, "%s: split before %s\n", __func__, st[count]->text());
+      LOG_FMT(LSPLIT, "%s(%d): split before %s\n", __func__, __LINE__, st[count]->text());
       split_before_chunk(chunk_get_next(st[count]));
    }
 
@@ -588,7 +591,7 @@ static void split_for_stmt(chunk_t *start)
    pc = open_paren;
    while ((pc = chunk_get_next(pc)) != start)
    {
-      if (pc->type == CT_COMMA && (pc->level == (open_paren->level + 1)))
+      if (chunk_is_token(pc, CT_COMMA) && (pc->level == (open_paren->level + 1)))
       {
          split_before_chunk(chunk_get_next(pc));
          if (!is_past_width(pc))
@@ -602,7 +605,7 @@ static void split_for_stmt(chunk_t *start)
    pc = open_paren;
    while ((pc = chunk_get_next(pc)) != start)
    {
-      if (pc->type == CT_ASSIGN && (pc->level == (open_paren->level + 1)))
+      if (chunk_is_token(pc, CT_ASSIGN) && (pc->level == (open_paren->level + 1)))
       {
          split_before_chunk(chunk_get_next(pc));
          if (!is_past_width(pc))
@@ -627,7 +630,7 @@ static void split_fcn_params_full(chunk_t *start)
    {
       LOG_FMT(LSPLIT, "%s(%d): %s, orig_col is %zu, level is %zu\n",
               __func__, __LINE__, fpo->text(), fpo->orig_col, fpo->level);
-      if (fpo->type == CT_FPAREN_OPEN && (fpo->level == start->level - 1))
+      if (chunk_is_token(fpo, CT_FPAREN_OPEN) && (fpo->level == start->level - 1))
       {
          break;  // opening parenthesis found. Issue #1020
       }
@@ -641,7 +644,7 @@ static void split_fcn_params_full(chunk_t *start)
       {
          break;
       }
-      if ((pc->level == (fpo->level + 1)) && pc->type == CT_COMMA)
+      if ((pc->level == (fpo->level + 1)) && chunk_is_token(pc, CT_COMMA))
       {
          split_before_chunk(chunk_get_next(pc));
       }
@@ -653,23 +656,25 @@ static void split_fcn_params(chunk_t *start)
 {
    LOG_FUNC_ENTRY();
    LOG_FMT(LSPLIT, "%s(%d): '%s'\n", __func__, __LINE__, start->text());
-
-   // Find the opening function parenthesis
    chunk_t *fpo = start;
-   LOG_FMT(LSPLIT, "%s(%d): Find the opening function parenthesis\n", __func__, __LINE__);
-   while (  ((fpo = chunk_get_prev(fpo)) != nullptr)
-         && fpo->type != CT_FPAREN_OPEN)
+   if (!chunk_is_token(start, CT_FPAREN_OPEN))
    {
-      // do nothing
-      LOG_FMT(LSPLIT, "%s(%d): '%s', orig_col is %zu, level is %zu\n",
-              __func__, __LINE__, fpo->text(), fpo->orig_col, fpo->level);
+      // Find the opening function parenthesis
+      LOG_FMT(LSPLIT, "%s(%d): Find the opening function parenthesis\n", __func__, __LINE__);
+      while (  ((fpo = chunk_get_prev(fpo)) != nullptr)
+            && chunk_is_not_token(fpo, CT_FPAREN_OPEN))
+      {
+         // do nothing
+         LOG_FMT(LSPLIT, "%s(%d): '%s', orig_col is %zu, level is %zu\n",
+                 __func__, __LINE__, fpo->text(), fpo->orig_col, fpo->level);
+      }
    }
 
    chunk_t *pc     = chunk_get_next_ncnl(fpo);
    size_t  min_col = pc->column;
 
    LOG_FMT(LSPLIT, "    mincol is %zu, max_width is %zu\n",
-           min_col, cpd.settings[UO_code_width].u - min_col);
+           min_col, options::code_width() - min_col);
 
    int cur_width = 0;
    int last_col  = -1;
@@ -696,13 +701,13 @@ static void split_fcn_params(chunk_t *start)
 
          LOG_FMT(LSPLIT, "%s(%d): last_col is %d\n",
                  __func__, __LINE__, last_col);
-         if (pc->type == CT_COMMA || pc->type == CT_FPAREN_CLOSE)
+         if (chunk_is_token(pc, CT_COMMA) || chunk_is_token(pc, CT_FPAREN_CLOSE))
          {
             cur_width--;
             LOG_FMT(LSPLIT, "%s(%d): cur_width is %d\n",
                     __func__, __LINE__, cur_width);
-            if (  ((last_col - 1) > static_cast<int>(cpd.settings[UO_code_width].u))
-               || pc->type == CT_FPAREN_CLOSE)
+            if (  ((last_col - 1) > static_cast<int>(options::code_width()))
+               || chunk_is_token(pc, CT_FPAREN_CLOSE))
             {
                break;
             }
@@ -718,7 +723,7 @@ static void split_fcn_params(chunk_t *start)
    {
       LOG_FMT(LSPLIT, "%s(%d): pc '%s', pc->level is %zu, prev '%s', prev->type is %s\n",
               __func__, __LINE__, pc->text(), pc->level, prev->text(), get_token_name(prev->type));
-      if (chunk_is_newline(prev) || prev->type == CT_COMMA)
+      if (chunk_is_newline(prev) || chunk_is_token(prev, CT_COMMA))
       {
          LOG_FMT(LSPLIT, "%s(%d): found at %zu\n",
                  __func__, __LINE__, prev->orig_col);
@@ -729,21 +734,21 @@ static void split_fcn_params(chunk_t *start)
       last_col -= pc->len();
       LOG_FMT(LSPLIT, "%s(%d): last_col is %d\n",
               __func__, __LINE__, last_col);
-      if (prev->type == CT_FPAREN_OPEN)
+      if (chunk_is_token(prev, CT_FPAREN_OPEN))
       {
          pc = chunk_get_next(prev);
-         if (!cpd.settings[UO_indent_paren_nl].b)
+         if (!options::indent_paren_nl())
          {
-            min_col = pc->brace_level * cpd.settings[UO_indent_columns].u + 1;
+            min_col = pc->brace_level * options::indent_columns() + 1;
             LOG_FMT(LSPLIT, "%s(%d): min_col is %zu\n",
                     __func__, __LINE__, min_col);
-            if (cpd.settings[UO_indent_continue].n == 0)
+            if (options::indent_continue() == 0)
             {
-               min_col += cpd.settings[UO_indent_columns].u;
+               min_col += options::indent_columns();
             }
             else
             {
-               min_col += abs(cpd.settings[UO_indent_continue].n);
+               min_col += abs(options::indent_continue());
             }
             LOG_FMT(LSPLIT, "%s(%d): min_col is %zu\n",
                     __func__, __LINE__, min_col);
@@ -758,7 +763,7 @@ static void split_fcn_params(chunk_t *start)
    }
    if (prev != nullptr && !chunk_is_newline(prev))
    {
-      LOG_FMT(LSPLIT, "%s(%d): -- ended on [%s] --\n",
+      LOG_FMT(LSPLIT, "%s(%d): -- ended on %s --\n",
               __func__, __LINE__, get_token_name(prev->type));
       LOG_FMT(LSPLIT, "%s(%d): min_col is %zu\n",
               __func__, __LINE__, min_col);
@@ -781,7 +786,7 @@ static void split_template(chunk_t *start)
    while ((prev = chunk_get_prev(prev)) != nullptr)
    {
       LOG_FMT(LSPLIT, "  %s(%d): prev '%s'\n", __func__, __LINE__, prev->text());
-      if (chunk_is_newline(prev) || prev->type == CT_COMMA)
+      if (chunk_is_newline(prev) || chunk_is_token(prev, CT_COMMA))
       {
          break;
       }
@@ -790,17 +795,17 @@ static void split_template(chunk_t *start)
    if (prev != nullptr && !chunk_is_newline(prev))
    {
       LOG_FMT(LSPLIT, "  %s(%d):", __func__, __LINE__);
-      LOG_FMT(LSPLIT, " -- ended on [%s] --\n", get_token_name(prev->type));
+      LOG_FMT(LSPLIT, " -- ended on %s --\n", get_token_name(prev->type));
       chunk_t *pc = chunk_get_next(prev);
       newline_add_before(pc);
       size_t  min_col = 1;
-      if (cpd.settings[UO_indent_continue].n == 0)
+      if (options::indent_continue() == 0)
       {
-         min_col += cpd.settings[UO_indent_columns].u;
+         min_col += options::indent_columns();
       }
       else
       {
-         min_col += abs(cpd.settings[UO_indent_continue].n);
+         min_col += abs(options::indent_continue());
       }
       reindent_line(pc, min_col);
       cpd.changes++;
